Type: Package
Package: tufte
Title: Tufte's Styles for R Markdown Documents
Version: 0.15.0
Authors@R: c(
    person("Yihui", "Xie", role = "aut", email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666")),
    person("Christophe", "Dervieux", role = c("ctb", "cre"), email = "cderv@posit.co", comment = c(ORCID = "0000-0003-4474-2498")),
    person("JJ", "Allaire", role = "aut"),
    person("Andrzej", "Oles", role = "ctb"),
    person("Dave", "Liepmann", role = "ctb", comment = "Tufte CSS in inst/rmarkdown/templates/tufte_html/resources"),
    person("Posit Software, PBC", role = c("cph", "fnd"), comment = c(ROR = "03wc8by49"))
  )
Description: Provides R Markdown output formats to use Tufte styles for
    PDF and HTML output.
License: GPL-3
URL: https://github.com/rstudio/tufte
BugReports: https://github.com/rstudio/tufte/issues
Imports: htmltools, knitr (>= 1.28), rmarkdown (>= 2.11), xfun (>=
        0.13)
Suggests: xml2, bookdown, covr, testthat (>= 3.1.0), tinytex, withr (>=
        2.3.0)
Config/testthat/edition: 3
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-05-21 15:08:54 UTC; chris
Author: Yihui Xie [aut] (ORCID: <https://orcid.org/0000-0003-0645-5666>),
  Christophe Dervieux [ctb, cre] (ORCID:
    <https://orcid.org/0000-0003-4474-2498>),
  JJ Allaire [aut],
  Andrzej Oles [ctb],
  Dave Liepmann [ctb] (Tufte CSS in
    inst/rmarkdown/templates/tufte_html/resources),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Christophe Dervieux <cderv@posit.co>
Repository: CRAN
Date/Publication: 2026-05-21 16:30:02 UTC

# Generated by roxygen2: do not edit by hand

export(margin_note)
export(newthought)
export(quote_footer)
export(sans_serif)
export(tufte_book)
export(tufte_book2)
export(tufte_handout)
export(tufte_handout2)
export(tufte_html)
export(tufte_html2)
importFrom(htmltools,htmlDependency)
importFrom(knitr,is_html_output)
importFrom(knitr,is_latex_output)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# tufte

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/tufte)](https://CRAN.R-project.org/package=tufte)
[![Downloads from the RStudio CRAN
mirror](https://cranlogs.r-pkg.org/badges/tufte)](https://cran.r-project.org/package=tufte)
[![R-CMD-check](https://github.com/rstudio/tufte/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/rstudio/tufte/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/rstudio/tufte/graph/badge.svg)](https://app.codecov.io/gh/rstudio/tufte)
<!-- badges: end -->

This R package provides a few R Markdown output formats that use the
Tufte style. See <https://rstudio.github.io/tufte/> for a comprehensive
example.

## Books

<a href="https://yihui.org/rmarkdown/tufte-handouts.html"><img src="https://yihui.org/rmarkdown/images/cover.png" alt="R Markdown: The Definitive Guide" class="book" height="400"/></a>

See about the Tufte Handouts format in R Markdown Definitive Guide

## Installation

You can install the last available released version from
[CRAN](https://cran.r-project.org/package=tufte)

``` r
install.packages('tufte')
```

You can also install the development version of **tufte** from
[GitHub](https://github.com/) with:

``` r
# install.packages("pak")
pak::pak("rstudio/tufte")
```

## Usage

The easiest way to make a new R Markdown document using Tufte style is
from within RStudio. Go to *File \> New File \> R Markdown \> From
template \> Tufte Handout*.

This can also be created from the command line using

``` r
rmarkdown::draft("tufte.Rmd", "tufte_html", "tufte")
```

## Getting help

There are two main places to get help:

1.  The [RStudio
    community](https://forum.posit.co/c/quarto-r-markdown/10) is a
    friendly place to ask any questions about rmarkdown and the R
    Markdown family of packages. Use tag **tufte** in your post.

2.  [Stack
    Overflow](https://stackoverflow.com/questions/tagged/r-markdown+tufte)
    is a great source of answers to common rmarkdown questions. It is
    also a great place to get help, once you have created a reproducible
    example that illustrates your problem.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# tufte

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/tufte)](https://CRAN.R-project.org/package=tufte)
[![Downloads from the RStudio CRAN
mirror](https://cranlogs.r-pkg.org/badges/tufte)](https://cran.r-project.org/package=tufte)
[![R-CMD-check](https://github.com/rstudio/tufte/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/rstudio/tufte/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/rstudio/tufte/graph/badge.svg)](https://app.codecov.io/gh/rstudio/tufte)
<!-- badges: end -->

This R package provides a few R Markdown output formats that use the
Tufte style. See <https://rstudio.github.io/tufte/> for a comprehensive
example.

## Books

<a href="https://bookdown.org/yihui/rmarkdown/tufte-handouts.html"><img src="https://bookdown.org/yihui/rmarkdown/images/cover.png" alt="R Markdown: The Definitive Guide" class="book" height="400"/></a>

See about the Tufte Handouts format in R Markdown Definitive Guide

## Installation

You can install the last available released version from
[CRAN](https://cran.r-project.org/package=tufte)

``` r
install.packages('tufte')
```

You can also install the development version of **tufte** from
[GitHub](https://github.com/) with:

``` r
# install.packages("pak")
pak::pak("rstudio/tufte")
```

## Usage

The easiest way to make a new R Markdown document using Tufte style is
from within RStudio. Go to *File \> New File \> R Markdown \> From
template \> Tufte Handout*.

This can also be created from the command line using

``` r
rmarkdown::draft("tufte.Rmd", "tufte_html", "tufte")
```

## Getting help

There are two main places to get help:

1.  The [RStudio
    community](https://forum.posit.co/c/quarto-r-markdown/10) is a
    friendly place to ask any questions about rmarkdown and the R
    Markdown family of packages. Use tag **tufte** in your post.

2.  [Stack
    Overflow](https://stackoverflow.com/questions/tagged/r-markdown+tufte)
    is a great source of answers to common rmarkdown questions. It is
    also a great place to get help, once you have created a reproducible
    example that illustrates your problem.

# Use to test pandoc availability or version lower than
skip_if_not_tinytex <- function() {
  if (!tinytex::is_tinytex()) skip("TinyTeX is not installed")
}

skip_if_not_pandoc <- function(ver = NULL) {
  if (!rmarkdown::pandoc_available(ver)) {
    msg <- if (is.null(ver)) {
      "Pandoc is not available"
    } else {
      sprintf("Version of Pandoc is lower than %s.", ver)
    }
    skip(msg)
  }
}

# Use to test version greater than
skip_if_pandoc <- function(ver = NULL) {
  if (rmarkdown::pandoc_available(ver)) {
    msg <- if (is.null(ver)) {
      "Pandoc is available"
    } else {
      sprintf("Version of Pandoc is greater than %s.", ver)
    }
    skip(msg)
  }
}

local_rmd_file <- function(..., .env = parent.frame()) {
  path <- withr::local_tempfile(.local_envir = .env, fileext = ".Rmd")
  xfun::write_utf8(c(...), path)
  path
}

local_render <- function(input, ..., .env = parent.frame()) {
  skip_if_not_pandoc()
  output_file <- withr::local_tempfile(.local_envir = .env)
  rmarkdown::render(input, output_file = output_file, quiet = TRUE, ...)
}

local_pandoc_convert <- function(
  text,
  from = "markdown",
  options = NULL,
  ...,
  .env = parent.frame()
) {
  skip_if_not_pandoc()
  rmd <- local_rmd_file(text)
  out <- withr::local_tempfile(.local_envir = .env)
  rmarkdown::pandoc_convert(
    rmd,
    from = from,
    output = out,
    options = c("--wrap", "preserve", options),
    ...
  )
  xfun::read_utf8(out)
}

.render_and_read <- function(input, ...) {
  skip_if_not_pandoc()
  res <- local_render(input, ...)
  xfun::read_utf8(res)
}

# Temporarily set entries in knitr::opts_knit for the duration of the calling
# frame; original values are restored via withr::defer.
local_knit_opts <- function(..., .env = parent.frame()) {
  new <- list(...)
  if (!length(new)) {
    return(invisible())
  }
  keys <- names(new)
  old <- stats::setNames(
    lapply(keys, function(k) knitr::opts_knit$get(k)),
    keys
  )
  knitr::opts_knit$set(new)
  withr::defer(knitr::opts_knit$set(old), envir = .env)
  invisible()
}

# Unit tests for quote_footer() ----

test_that("quote_footer() produces styled <footer> for non-tufte HTML output", {
  local_mocked_bindings(
    is_html_output = function(...) TRUE,
    is_latex_output = function(...) FALSE
  )

  result <- quote_footer("Author Name")
  expect_match(result, "<footer")
  expect_match(result, 'class="blockquote-footer"')
  expect_match(result, "text-align: right")
  expect_match(result, "Author Name")
})

test_that("quote_footer() strips leading --- in non-tufte HTML", {
  local_mocked_bindings(
    is_html_output = function(...) TRUE,
    is_latex_output = function(...) FALSE
  )
  result <- quote_footer("--- Author Name")
  # Bootstrap CSS injects em-dash via ::before, so caller's --- is dropped.
  expect_no_match(result, "---", fixed = TRUE)
  expect_match(result, 'class="blockquote-footer"')
  expect_match(result, "Author Name")
})

test_that("quote_footer() inserts em-dash glyph in tufte HTML when caller wrote ---", {
  local_mocked_bindings(
    is_html_output = function(...) TRUE,
    is_latex_output = function(...) FALSE
  )
  local_knit_opts(tufte.format = "html")
  result <- quote_footer("--- Author Name")
  # tufte.css has no ::before; the glyph + nbsp is injected in markup.
  expect_match(result, "— Author Name")
  expect_no_match(result, 'class="blockquote-footer"', fixed = TRUE)
  expect_match(result, "text-align: right")
})

test_that("quote_footer() in tufte HTML without --- preserves text as-is", {
  local_mocked_bindings(
    is_html_output = function(...) TRUE,
    is_latex_output = function(...) FALSE
  )
  local_knit_opts(tufte.format = "html")
  result <- quote_footer("Author Name")
  expect_no_match(result, "—", fixed = TRUE)
  expect_no_match(result, 'class="blockquote-footer"', fixed = TRUE)
  expect_match(result, "Author Name")
})

test_that("quote_footer() produces \\hfill for LaTeX output", {
  local_mocked_bindings(
    is_html_output = function(...) FALSE,
    is_latex_output = function(...) TRUE
  )
  expect_equal(quote_footer("Author Name"), "\\hfill Author Name")
  # Leading --- is preserved so pandoc smart-punct converts to em-dash.
  expect_equal(quote_footer("--- Author Name"), "\\hfill --- Author Name")
})

test_that("quote_footer() warns for unsupported output", {
  local_mocked_bindings(
    is_html_output = function(...) FALSE,
    is_latex_output = function(...) FALSE
  )
  local_mocked_bindings(pandoc_to = function(...) "docx", .package = "knitr")
  expect_warning(
    result <- quote_footer("Author Name"),
    "only works for HTML and LaTeX"
  )
  expect_equal(result, "Author Name")
})

test_that("quote_footer() is silent outside a pandoc context", {
  local_mocked_bindings(
    is_html_output = function(...) FALSE,
    is_latex_output = function(...) FALSE
  )
  local_mocked_bindings(pandoc_to = function(...) NULL, .package = "knitr")
  expect_no_warning(result <- quote_footer("Author Name"))
  expect_equal(result, "Author Name")
})

# Integration tests: render quote_footer() in non-tufte formats ----

test_that("quote_footer() renders in html_document with inline styles", {
  skip_if_not_pandoc()
  skip_if_not_installed("xml2")
  rmd <- local_rmd_file(
    "---",
    "output: html_document",
    "---",
    "",
    "> A quote",
    ">",
    # Bootstrap CSS injects the em-dash via ::before; the helper should strip
    # the leading "---" so it is not doubled.
    "> `r tufte::quote_footer('--- Author')`"
  )
  html <- .render_and_read(rmd)
  doc <- xml2::read_html(paste(html, collapse = "\n"))
  footer <- xml2::xml_find_first(doc, "//blockquote//footer")
  expect_false(is.na(footer))
  expect_match(xml2::xml_attr(footer, "style"), "text-align: right")
  expect_match(xml2::xml_attr(footer, "class"), "blockquote-footer")
  expect_match(xml2::xml_text(footer), "Author")
  # No literal "---" in the rendered HTML between <footer> tags.
  expect_no_match(xml2::xml_text(footer), "---", fixed = TRUE)
})

test_that("quote_footer() renders in html_document with bslib BS4 theme", {
  skip_if_not_pandoc()
  skip_if_not_installed("xml2")
  skip_if_not_installed("bslib")
  rmd <- local_rmd_file(
    "---",
    "output:",
    "  html_document:",
    "    theme:",
    "      version: 4",
    "---",
    "",
    "> A quote",
    ">",
    "> `r tufte::quote_footer('Author')`"
  )
  html <- .render_and_read(rmd)
  doc <- xml2::read_html(paste(html, collapse = "\n"))
  footer <- xml2::xml_find_first(doc, "//blockquote//footer")
  expect_false(is.na(footer))
  expect_match(xml2::xml_attr(footer, "class"), "blockquote-footer")
  expect_match(xml2::xml_attr(footer, "style"), "text-align: right")
})

test_that("quote_footer() renders in pdf_document", {
  skip_if_not_pandoc()
  skip_if_not_tinytex()
  rmd <- local_rmd_file(
    "---",
    "output: pdf_document",
    "---",
    "",
    "> A quote",
    ">",
    "> `r tufte::quote_footer('Author')`"
  )
  expect_no_error(local_render(rmd))
})

test_that("quote_footer() renders in beamer_presentation", {
  skip_if_not_pandoc()
  skip_if_not_tinytex()
  rmd <- local_rmd_file(
    "---",
    "output: beamer_presentation",
    "---",
    "",
    "## Slide",
    "",
    "> A quote",
    ">",
    "> `r tufte::quote_footer('Author')`"
  )
  expect_no_error(local_render(rmd))
})

test_that("quote_footer() preserves citation for LaTeX/natbib (#73)", {
  # Older pandoc consumed `\hfill` + rest of line as raw LaTeX, swallowing the
  # markdown citation that followed. Lock current behaviour: citation is still
  # parsed when it follows quote_footer()'s `\hfill` output.
  skip_if_not_pandoc()
  local_mocked_bindings(
    is_html_output = function(...) FALSE,
    is_latex_output = function(...) TRUE
  )
  footer <- quote_footer("--- [@dorian10: 3]")
  tex <- local_pandoc_convert(footer, to = "latex", options = "--natbib")
  tex <- paste(tex, collapse = "\n")
  expect_match(tex, "\\\\hfill")
  expect_match(tex, "\\\\citep\\[3\\]\\{dorian10\\}")
})

test_that("quote_footer() still works in tufte_html", {
  skip_if_not_pandoc()
  skip_if_not_installed("xml2")
  rmd <- local_rmd_file(
    "---",
    "output: tufte::tufte_html",
    "---",
    "",
    "> A quote",
    ">",
    # Leading "---" should render as an em-dash glyph in tufte HTML too,
    # since tufte.css does not inject one via ::before.
    "> `r tufte::quote_footer('--- Author')`"
  )
  html <- .render_and_read(rmd)
  doc <- xml2::read_html(paste(html, collapse = "\n"))
  footer <- xml2::xml_find_first(doc, "//blockquote//footer")
  expect_false(is.na(footer))
  txt <- xml2::xml_text(footer)
  expect_match(txt, "—")
  expect_match(txt, "Author")
  expect_no_match(txt, "---", fixed = TRUE)
})

test_that("add marginnote", {
  expect_snapshot(marginnote_html("text"))
  expect_snapshot(marginnote_html("text", "#"))
})

expect_refs_margin <- function(
  moved = FALSE,
  options = NULL,
  ...,
  variant = NULL
) {
  rmd <- test_path("resources/margins_references.Rmd")
  out <- withr::local_tempfile(fileext = ".html")
  rmd_temp <- withr::local_tempfile(fileext = ".Rmd")
  xfun::write_utf8(
    knitr::knit_expand(rmd, linked = if (moved) "yes" else "no"),
    rmd_temp
  )
  rmarkdown::pandoc_convert(
    basename(rmd_temp),
    "html4",
    "markdown",
    output = out,
    citeproc = TRUE,
    verbose = FALSE,
    wd = dirname(rmd_temp),
    options = c("--wrap", "preserve", options),
    ...
  )
  x <- xfun::read_utf8(out)
  expect_snapshot(margin_references(x), variant = variant)
}

citeproc_variant <- function() {
  if (!rmarkdown::pandoc_available("2.11")) {
    "pandoc-citeproc"
  } else if (!rmarkdown::pandoc_available("2.14.1")) {
    # new citeproc creates links on author
    "new-citeproc-post-2.14.0.2"
  } else if (!rmarkdown::pandoc_available("3.1.8")) {
    # Pandoc 2.14.1 fixed  that
    "new-citeproc-post-2.14.1"
  } else if (!rmarkdown::pandoc_available("3.8")) {
    "new-citeproc-post-3.1.8"
  } else {
    "new-citeproc-post-3.8"
  }
}

pandoc_variant <- citeproc_variant()

test_that("put references in margin when link-citations: yes", {
  skip_on_cran()
  skip_if_not_pandoc()
  expect_refs_margin(moved = TRUE, variant = pandoc_variant)
  expect_refs_margin(moved = FALSE, variant = pandoc_variant)
})

test_that("put references in margin when link-citations: yes using csl", {
  skip_on_cran() # requires recent Pandoc
  skip_if_not_pandoc("2.11")
  skip_if_offline("zotero.org")
  expect_refs_margin(
    moved = TRUE,
    variant = pandoc_variant,
    c("--csl", "https://www.zotero.org/styles/apa-6th-edition")
  )
  expect_refs_margin(
    moved = TRUE,
    variant = pandoc_variant,
    c("--csl", "https://www.zotero.org/styles/chicago-author-date-16th-edition")
  )
})

test_that("nocite references survive margin_references() (issue #35)", {
  skip_on_cran()
  skip_if_not_pandoc()
  rmd <- local_rmd_file(
    "---",
    "title: test",
    "output: tufte::tufte_html",
    "bibliography: refs.bib",
    "link-citations: yes",
    "nocite: |",
    "  @nocite_only",
    "---",
    "",
    "See @cited_in_text for details."
  )
  # Create a minimal .bib next to the Rmd
  bib <- file.path(dirname(rmd), "refs.bib")
  xfun::write_utf8(
    c(
      "@article{cited_in_text,",
      "  author = {Smith, John},",
      "  title = {Cited Article},",
      "  journal = {J. Examples},",
      "  year = {2020}",
      "}",
      "@article{nocite_only,",
      "  author = {Doe, Jane},",
      "  title = {Nocite Article},",
      "  journal = {J. Nocite},",
      "  year = {2019}",
      "}"
    ),
    bib
  )
  withr::defer(unlink(bib))
  html <- .render_and_read(rmd)
  # The nocite-only reference must appear somewhere in the output
  expect_true(
    any(grepl("ref-nocite_only", html)),
    info = "nocite entry should not be dropped when link-citations: yes (issue #35)"
  )
  # The in-text reference should also still be present (as margin note)
  expect_true(
    any(grepl("Cited Article|ref-cited_in_text", html)),
    info = "in-text citation should still appear"
  )
})

test_that("fig.margin=TRUE works with fig.align set (issue #54)", {
  skip_on_cran()
  skip_if_not_pandoc()
  rmd <- local_rmd_file(
    "---",
    "title: test",
    "output: tufte::tufte_html",
    "---",
    "",
    '```{r fig-margin-align, fig.margin=TRUE, fig.align="center", fig.cap="test cap"}',
    "plot(1)",
    "```"
  )
  html <- .render_and_read(rmd)
  # The margin figure wrapper must be present
  margin_lines <- grep("marginnote shownote", html, value = TRUE)
  expect_true(
    length(margin_lines) > 0,
    info = "fig.margin=TRUE with fig.align should still produce marginnote wrapper"
  )
  # The raw <div class="figure" style="text-align: ..."> should NOT appear
  # (it should be commented out inside the marginnote wrapper)
  raw_div <- grep(
    '<div class="figure" style=',
    html,
    fixed = TRUE,
    value = TRUE
  )
  expect_true(
    all(grepl("<!--", raw_div)),
    info = "fig div with style should be inside HTML comment when fig.margin=TRUE"
  )
})

test_that("fig.fullwidth=TRUE works with fig.align set", {
  skip_on_cran()
  skip_if_not_pandoc()
  rmd <- local_rmd_file(
    "---",
    "title: test",
    "output: tufte::tufte_html",
    "---",
    "",
    '```{r fig-full-align, fig.fullwidth=TRUE, fig.align="center", fig.cap="test cap"}',
    "plot(1)",
    "```"
  )
  html <- .render_and_read(rmd)
  fullwidth_lines <- grep("figure fullwidth", html, value = TRUE)
  expect_true(
    length(fullwidth_lines) > 0,
    info = "fig.fullwidth=TRUE with fig.align should produce fullwidth class"
  )
})

# tufte_html2 (bookdown wrapper) -------------------------------------------

test_that("tufte_html2() renders", {
  skip_on_cran()
  skip_if_not_pandoc()
  skip_if_not_installed("bookdown")
  rmd <- local_rmd_file(
    "---",
    "title: test",
    "output: tufte::tufte_html2",
    "---",
    "",
    "Hello world."
  )
  html <- .render_and_read(rmd)
  expect_true(length(html) > 0)
})

test_that("tufte_html2() resolves text references in fig.cap (#60)", {
  skip_on_cran()
  skip_if_not_pandoc()
  skip_if_not_installed("bookdown")
  rmd <- local_rmd_file(
    "---",
    "output: tufte::tufte_html2",
    "---",
    "",
    "(ref:cars-cap) A plot of the [cars data set](https://example.org).",
    "",
    "```{r cars-plot, fig.cap='(ref:cars-cap)', echo=FALSE}",
    "plot(cars)",
    "```"
  )
  html <- paste(.render_and_read(rmd), collapse = "\n")
  # Primary assertion: a regression that stops resolving (ref:label) in
  # captions would leave the literal label in the rendered HTML. The
  # link/href assertion alone is not enough because the (ref:) definition
  # paragraph also contains the rendered link.
  expect_false(grepl("(ref:cars-cap)", html, fixed = TRUE))
  expect_match(html, '<a href="https://example\\.org">cars data set</a>')
  # Raw markdown link syntax should NOT appear in resolved output
  expect_false(grepl("](https://example.org)", html, fixed = TRUE))
})

# footnote parsing ---------------------------------------------------------

test_that("footnotes are correctly parsed", {
  skip_on_cran()
  skip_if_not_pandoc()
  pandoc_html <- local_pandoc_convert(
    "Here is some text^[This should be a sidenote].",
    to = "html4"
  )
  expect_identical(
    parse_footnotes(pandoc_html),
    list(items = "This should be a sidenote", range = 2:7)
  )
})

test_that("tufte_handout() does not warn about crop tools", {
  expect_no_warning(tufte_handout())
})

test_that("tufte_book() does not warn about crop tools", {
  expect_no_warning(tufte_book())
})

test_that("fig_crop = 'auto' disables crop hook when tools are missing", {
  # Crop-tool detection (pdfcrop + ghostscript) varies by platform and
  # CRAN check environment, so query rmarkdown's own resolver to decide
  # whether the preconditions for this test hold. If rmarkdown would
  # itself set a crop hook for fig_crop = "auto", the test's premise
  # does not apply on this machine.
  baseline <- rmarkdown::knitr_options_pdf(4, 2.5, "auto", "pdf")
  skip_if(
    !is.null(baseline$knit_hooks$crop),
    "crop tools must be missing for this test"
  )
  fmt <- tufte_handout()
  expect_null(fmt$knitr$knit_hooks$crop)
})

test_that("tufte_handout renders without pdfcrop warnings", {
  skip_on_cran()
  skip_if_not_pandoc()
  rmd <- local_rmd_file(c(
    "---",
    "title: test",
    "output: tufte::tufte_handout",
    "---",
    "",
    "Hello world."
  ))
  # Suppress unrelated LaTeX warnings, but fail on any crop-related ones
  withCallingHandlers(
    local_render(rmd),
    warning = function(w) {
      msg <- conditionMessage(w)
      if (grepl("pdfcrop|crop_tools|ghostscript", msg)) {
        fail(paste("Unexpected crop tool warning:", msg))
      }
      invokeRestart("muffleWarning")
    }
  )
  succeed()
})

.onLoad <- function(lib, pkg) {
  # this engine will be overridden in tufte_html/tufte_handout formats
  knitr::knit_engines$set(marginfigure = function(options) {
    'Placeholder (you should not see this)'
  })
}

#' @details `newthought()` can be used in inline R expressions in R
#'   Markdown
#'   ```r
#'   `r newthought(Some text)`
#'   ```
#'   and it works for both
#'   HTML (\samp{<span class="newthought">text</span>}) and PDF
#'   (\samp{\\newthought{text}}) output.
#' @param text A character string to be presented as a \dQuote{new thought}
#'   (using small caps), or a margin note, or a footer of a quote
#' @rdname tufte_handout
#' @export
#' @examples newthought("In this section")
newthought <- function(text) {
  if (is_html_output()) {
    sprintf('<span class="newthought">%s</span>', text)
  } else if (is_latex_output()) {
    sprintf("\\newthought{%s}", text)
  } else {
    sprintf('<span style="font-variant:small-caps;">%s</span>', text)
  }
}

#' @details `margin_note()` can be used in inline R expressions to write a
#'   margin note (like a sidenote but not numbered).
#' @param icon A character string to indicate there is a hidden margin note when
#'   the page width is too narrow (by default it is a circled plus sign)
#' @rdname tufte_handout
#' @importFrom knitr is_html_output is_latex_output
#' @export
margin_note <- function(text, icon = "&#8853;") {
  if (is_html_output()) {
    marginnote_html(sprintf('<span class="marginnote">%s</span>', text), icon)
  } else if (is_latex_output()) {
    sprintf("\\marginnote{%s}", text)
  } else {
    warning("marginnote() only works for HTML and LaTeX output", call. = FALSE)
    text
  }
}

#' @details `quote_footer()` formats text as the footer of a quote. It works in
#'   any HTML or LaTeX output format, not only tufte formats, and tries to
#'   render `text` consistently across formats:
#'
#'   * If `text` starts with `"---"` (or an em-dash), an em-dash + non-breaking
#'     space is rendered before the rest of the text in every output. The
#'     leading `"---"` is stripped from `text` to avoid duplication when the
#'     destination format would otherwise inject its own em-dash.
#'
#'   * For tufte HTML, `text` is wrapped in a `<footer>` element styled by
#'     `tufte.css`. For non-tufte HTML (e.g. `rmarkdown::html_document()`,
#'     including `bslib` Bootstrap 4 and 5 themes), `text` is wrapped in a
#'     `<footer class="blockquote-footer">` so that Bootstrap's
#'     `.blockquote-footer` styling (including the `::before` em-dash) applies.
#'     An inline `text-align: right` is added in both cases.
#'
#'   * For LaTeX output (tufte or non-tufte), `text` is prepended with
#'     \samp{\\hfill} to right-align it. Pandoc's smart-punctuation extension
#'     converts a leading `"---"` to an em-dash glyph.
#'
#'   `quote_footer()` detects whether the active output is a tufte format via
#'   the `tufte.format` entry that the tufte output formats register in
#'   `knitr::opts_knit`.
#' @rdname tufte_handout
#' @export
quote_footer <- function(text) {
  dash_pattern <- "^\\s*(?:---|\u2014)\\s*"
  has_dash <- grepl(dash_pattern, text)
  stripped <- sub(dash_pattern, "", text)
  in_tufte <- !is.null(knitr::opts_knit$get("tufte.format"))

  if (is_html_output()) {
    if (in_tufte) {
      # tufte.css does not inject an em-dash via ::before, so when the caller
      # requested one with leading "---", insert the glyph ourselves.
      prefix <- if (has_dash) "\u2014\u00a0" else ""
      sprintf(
        '<footer style="text-align: right;">%s%s</footer>',
        prefix,
        stripped
      )
    } else {
      # Bootstrap 3/4/5 inject "em-dash + nbsp" via ::before on either
      # `blockquote footer` (BS3) or `.blockquote-footer` (BS4/5); the strip
      # avoids a doubled em-dash when callers follow the documented
      # `quote_footer('--- Author')` pattern.
      sprintf(
        '<footer class="blockquote-footer" style="text-align: right;">%s</footer>',
        stripped
      )
    }
  } else if (is_latex_output()) {
    # Restore the caller-supplied "---" after \hfill so pandoc's smart-punct
    # extension converts it to an em-dash glyph, mirroring the HTML behaviour.
    prefix <- if (has_dash) "--- " else ""
    sprintf("\\hfill %s%s", prefix, stripped)
  } else {
    if (!is.null(knitr::pandoc_to())) {
      warning(
        "quote_footer() only works for HTML and LaTeX output",
        call. = FALSE
      )
    }
    text
  }
}

#' @details `sans_serif()` applies sans-serif fonts to `text`.
#' @rdname tufte_handout
#' @export
sans_serif <- function(text) {
  if (is_html_output()) {
    sprintf('<span class="sans">%s</span>', text)
  } else if (is_latex_output()) {
    sprintf("\\textsf{%s}", text)
  } else {
    warning("sans_serif() only works for HTML and LaTeX output", call. = FALSE)
    text
  }
}

check_bookdown <- function() {
  if (!xfun::pkg_available("bookdown")) {
    stop(
      "The 'bookdown' package is required for tufte_handout2(), tufte_book2(), and tufte_html2(). ",
      "Install it with install.packages('bookdown').",
      call. = FALSE
    )
  }
}

# Decide whether the current chunk has a `fig.pos` value that should win
# over `margin_fig_pos`. The plot hook receives merged options
# (chunk-level + global defaults), so `options$fig.pos` alone cannot tell
# the two apart. Two independent signals together cover the cases:
#
#   1. The raw chunk header (`options$params.src`) contains a literal
#      `fig.pos=` -- this catches the direct-write case even when the
#      chunk value happens to equal the global default.
#   2. The merged `options$fig.pos` differs from the global default
#      `knitr::opts_chunk$get("fig.pos")` -- this catches indirect
#      chunk-level settings via `opts.label` / `opts_template`, which
#      never appear in `params.src`.
#
# Used by the margin-figure precedence rule for `margin_fig_pos` (#62).
chunk_overrides_fig_pos <- function(options) {
  params_src <- options$params.src
  if (!is.null(params_src) && grepl("(^|,)\\s*fig\\.pos\\s*=", params_src)) {
    return(TRUE)
  }
  !identical(options$fig.pos, knitr::opts_chunk$get("fig.pos"))
}

devtools_loaded <- function(x) {
  if (!x %in% loadedNamespaces()) {
    return(FALSE)
  }
  ns <- .getNamespace(x)
  !is.null(ns$.__DEVTOOLS__)
}

pkg_file <- function(..., package = "tufte", mustWork = FALSE) {
  if (devtools_loaded(package)) {
    file.path(find.package(package), "inst", ...)
  } else {
    system.file(..., package = package, mustWork = mustWork)
  }
}

template_resources <- function(name, ...) {
  pkg_file("rmarkdown", "templates", name, "resources", ...)
}

gsub_fixed <- function(...) gsub(..., fixed = TRUE)

pandoc2.0 <- function() rmarkdown::pandoc_available("2.0")

# add --wrap=preserve to pandoc args for pandoc 2.0:
# https://github.com/rstudio/bookdown/issues/504
# https://github.com/rstudio/tufte/issues/115
add_wrap_preserve <- function(args, pandoc2 = pandoc2.0) {
  if (pandoc2 && !length(grep("--wrap", args))) {
    c("--wrap", "preserve", args)
  } else {
    args
  }
}

# pkg_file() ---------------------------------------------------------------

test_that("pkg_file() resolves to inst/ when loaded via pkgload", {
  patches_dir <- pkg_file("rmarkdown", "templates", "tufte_handout", "patches")
  expect_true(dir.exists(patches_dir))
  expect_true(file.exists(file.path(patches_dir, "tufte-common.def")))
  if (devtools_loaded("tufte")) {
    expect_match(patches_dir, file.path("inst", "rmarkdown"), fixed = TRUE)
  }
})

# PDF rendering ------------------------------------------------------------

local_render_pdf <- function(input, .env = parent.frame()) {
  skip_if_not_pandoc()
  skip_if_not_tinytex()
  output_file <- withr::local_tempfile(.local_envir = .env, fileext = ".pdf")
  # clean = FALSE preserves the .log file for post-render inspection.
  # Muffle bibentry warnings: tufte-latex uses \nobibliography* which triggers
  # advisories when no \bibliography follows. This is inherent to the class
  # and unrelated to what these tests exercise.
  withCallingHandlers(
    rmarkdown::render(
      input,
      output_file = output_file,
      quiet = TRUE,
      clean = FALSE
    ),
    warning = function(w) {
      if (
        grepl(
          "bibentry|nobibliography|Marginpar",
          conditionMessage(w),
          ignore.case = TRUE
        )
      ) {
        invokeRestart("muffleWarning")
      }
    }
  )
}

test_that("tufte_handout renders a basic PDF without error (#127)", {
  skip_on_cran()
  rmd <- local_rmd_file(
    "---",
    "output: tufte::tufte_handout",
    "---",
    "",
    "Hello world.",
    .env = parent.frame()
  )
  output <- local_render_pdf(rmd)
  expect_true(file.exists(output))
})

# margin_fig_pos chunk option (#62) ----------------------------------------

local_render_tex <- function(rmd_lines, .env = parent.frame()) {
  skip_if_not_pandoc()
  skip_if_not_tinytex()
  rmd <- local_rmd_file(rmd_lines, .env = .env)
  output <- withr::local_tempfile(.local_envir = .env, fileext = ".pdf")
  withCallingHandlers(
    rmarkdown::render(rmd, output_file = output, quiet = TRUE, clean = FALSE),
    warning = function(w) {
      if (
        grepl(
          "bibentry|nobibliography|Marginpar",
          conditionMessage(w),
          ignore.case = TRUE
        )
      ) {
        invokeRestart("muffleWarning")
      }
    }
  )
  tex_file <- xfun::with_ext(output, "tex")
  if (!file.exists(tex_file)) {
    skip("keep_tex did not produce a .tex file")
  }
  xfun::read_utf8(tex_file)
}

test_that("margin_fig_pos via YAML sets the marginfigure offset (#62)", {
  skip_on_cran()
  tex <- local_render_tex(c(
    "---",
    "output:",
    "  tufte::tufte_handout:",
    "    keep_tex: true",
    "    margin_fig_pos: '0.5cm'",
    "---",
    "",
    "```{r fig.margin=TRUE, fig.cap='test', echo=FALSE}",
    "plot(1:5)",
    "```",
    "",
    "Text after."
  ))
  marginfig <- grep("\\\\begin\\{marginfigure\\}", tex, value = TRUE)
  expect_length(marginfig, 1)
  expect_match(marginfig, "[0.5cm]", fixed = TRUE)
})

test_that("margin_fig_pos via opts_chunk$set works (#62)", {
  skip_on_cran()
  tex <- local_render_tex(c(
    "---",
    "output:",
    "  tufte::tufte_handout:",
    "    keep_tex: true",
    "---",
    "",
    "```{r setup, include=FALSE}",
    "knitr::opts_chunk$set(margin_fig_pos = '0.5cm')",
    "```",
    "",
    "```{r fig.margin=TRUE, fig.cap='test', echo=FALSE}",
    "plot(1:5)",
    "```",
    "",
    "Text after."
  ))
  marginfig <- grep("\\\\begin\\{marginfigure\\}", tex, value = TRUE)
  expect_length(marginfig, 1)
  expect_match(marginfig, "[0.5cm]", fixed = TRUE)
})

test_that("fig.pos overrides margin_fig_pos on a specific chunk (#62)", {
  skip_on_cran()
  tex <- local_render_tex(c(
    "---",
    "output:",
    "  tufte::tufte_handout:",
    "    keep_tex: true",
    "    margin_fig_pos: '0.5cm'",
    "---",
    "",
    "```{r fig.margin=TRUE, fig.pos='2cm', fig.cap='test', echo=FALSE}",
    "plot(1:5)",
    "```",
    "",
    "Text after."
  ))
  marginfig <- grep("\\\\begin\\{marginfigure\\}", tex, value = TRUE)
  expect_length(marginfig, 1)
  # fig.pos takes precedence
  expect_match(marginfig, "[2cm]", fixed = TRUE)
})

test_that("margin_fig_pos does not affect regular figures (#62)", {
  skip_on_cran()
  tex <- local_render_tex(c(
    "---",
    "output:",
    "  tufte::tufte_handout:",
    "    keep_tex: true",
    "    margin_fig_pos: '0.5cm'",
    "---",
    "",
    "```{r fig.cap='regular figure', echo=FALSE}",
    "plot(1:5)",
    "```",
    "",
    "Text after."
  ))
  # Regular figure should NOT have the margin offset
  figure_lines <- grep("\\\\begin\\{figure\\}", tex, value = TRUE)
  expect_false(any(grepl("0.5cm", figure_lines, fixed = TRUE)))
})

test_that("margin_fig_pos wins over a global opts_chunk$set(fig.pos=) (#62)", {
  skip_on_cran()
  # Reproduces the pre-fix bug: a global `fig.pos = "htbp"` (intended for
  # regular figures) used to leak onto margin chunks via merged-options
  # lookup, producing `\begin{marginfigure}[htbp]` and a LaTeX error.
  tex <- local_render_tex(c(
    "---",
    "output:",
    "  tufte::tufte_handout:",
    "    keep_tex: true",
    "    margin_fig_pos: '0.5cm'",
    "---",
    "",
    "```{r setup, include=FALSE}",
    "knitr::opts_chunk$set(fig.pos = 'htbp')",
    "```",
    "",
    "```{r fig.margin=TRUE, fig.cap='margin', echo=FALSE}",
    "plot(1:5)",
    "```",
    "",
    "```{r fig.cap='regular', echo=FALSE}",
    "plot(1:5)",
    "```"
  ))
  marginfig <- grep("\\\\begin\\{marginfigure\\}", tex, value = TRUE)
  expect_length(marginfig, 1)
  # Margin figure must use the length, not the global placement specifier
  expect_match(marginfig, "[0.5cm]", fixed = TRUE)
  expect_false(any(grepl("htbp", marginfig, fixed = TRUE)))
  # Regular figure must still receive the global placement specifier
  figure_lines <- grep("\\\\begin\\{figure\\}", tex, value = TRUE)
  expect_true(any(grepl("htbp", figure_lines, fixed = TRUE)))
})

test_that("opts_template fig.pos overrides margin_fig_pos (#62)", {
  skip_on_cran()
  # Indirect chunk-level fig.pos (via opts_template + opts.label) never
  # appears in `params.src`, so the params.src-literal check alone would
  # miss it. The hook must also notice that `options$fig.pos` differs from
  # the global knitr default and respect the chunk's intent.
  tex <- local_render_tex(c(
    "---",
    "output:",
    "  tufte::tufte_handout:",
    "    keep_tex: true",
    "    margin_fig_pos: '0.5cm'",
    "---",
    "",
    "```{r setup, include=FALSE}",
    "knitr::opts_template$set(margin_at_1 = list(fig.margin = TRUE, fig.pos = '1cm'))",
    "```",
    "",
    "```{r opts.label='margin_at_1', fig.cap='templated margin', echo=FALSE}",
    "plot(1:5)",
    "```"
  ))
  marginfig <- grep("\\\\begin\\{marginfigure\\}", tex, value = TRUE)
  expect_length(marginfig, 1)
  # opts_template value (1cm) must win, NOT the format-level margin_fig_pos
  expect_match(marginfig, "[1cm]", fixed = TRUE)
  expect_false(any(grepl("0.5cm", marginfig, fixed = TRUE)))
})

# tufte_handout2 / tufte_book2 (bookdown wrappers, issue #60) ---------------

test_that("tufte_handout2() renders a basic PDF", {
  skip_on_cran()
  skip_if_not_installed("bookdown")
  rmd <- local_rmd_file(
    "---",
    "output: tufte::tufte_handout2",
    "---",
    "",
    "Hello world."
  )
  output <- local_render_pdf(rmd)
  expect_true(file.exists(output))
})

test_that("tufte_handout2() resolves text references in fig.cap (#60)", {
  skip_on_cran()
  skip_if_not_installed("bookdown")
  tex <- local_render_tex(c(
    "---",
    "output:",
    "  tufte::tufte_handout2:",
    "    keep_tex: true",
    "---",
    "",
    "(ref:cars-cap) A plot of the [cars data set](https://stat.ethz.ch/R-manual/R-devel/library/datasets/html/cars.html).",
    "",
    "```{r cars-plot, fig.cap='(ref:cars-cap)', echo=FALSE}",
    "plot(cars)",
    "```"
  ))
  cap_lines <- grep("\\\\caption", tex, value = TRUE)
  expect_true(
    any(grepl("\\\\href\\{", cap_lines)),
    info = "fig.cap should contain \\href from resolved text reference"
  )
  # Raw markdown link syntax should NOT appear
  expect_false(
    any(grepl("](https://", cap_lines, fixed = TRUE)),
    info = "Raw markdown link should not appear in \\caption"
  )
})

test_that("tufte_book2() renders a basic PDF", {
  skip_on_cran()
  skip_if_not_installed("bookdown")
  rmd <- local_rmd_file(
    "---",
    "output: tufte::tufte_book2",
    "---",
    "",
    "Hello world."
  )
  output <- local_render_pdf(rmd)
  expect_true(file.exists(output))
})

test_that("check_bookdown() errors when bookdown is not available", {
  skip_on_cran()
  local_mocked_bindings(
    pkg_available = function(...) FALSE,
    .package = "xfun"
  )
  expect_error(check_bookdown(), "bookdown.*required")
})

# quote_footer() with natbib citation (#73) -------------------------------

# Renders an Rmd with a bibliography next to it in a tempdir so YAML's
# relative `bibliography:` path resolves. Returns the .tex content for
# inline assertions.
local_render_natbib_tex <- function(format, .env = parent.frame()) {
  skip_if_not_pandoc()
  skip_if_not_tinytex()
  tmpdir <- withr::local_tempdir(.local_envir = .env)
  bib_src <- test_path("resources", "refs-natbib.bib")
  bib_dest <- file.path(tmpdir, "refs-natbib.bib")
  file.copy(bib_src, bib_dest)
  rmd <- file.path(tmpdir, "doc.Rmd")
  xfun::write_utf8(
    c(
      "---",
      "output:",
      sprintf("  %s:", format),
      "    citation_package: natbib",
      "    keep_tex: true",
      "bibliography: refs-natbib.bib",
      "---",
      "",
      if (format == "beamer_presentation") "## Slide" else NULL,
      "",
      "> \"A test quote.\"",
      ">",
      "> `r tufte::quote_footer('--- [@dorian10: 3]')`"
    ),
    rmd
  )
  output <- file.path(tmpdir, "doc.pdf")
  withCallingHandlers(
    rmarkdown::render(rmd, output_file = output, quiet = TRUE, clean = FALSE),
    warning = function(w) {
      if (
        grepl(
          "bibentry|nobibliography",
          conditionMessage(w),
          ignore.case = TRUE
        )
      ) {
        invokeRestart("muffleWarning")
      }
    }
  )
  expect_true(file.exists(output))
  tex_file <- xfun::with_ext(output, "tex")
  skip_if(!file.exists(tex_file), "keep_tex did not produce a .tex file")
  xfun::read_utf8(tex_file)
}

test_that("tufte_handout renders natbib citation inside quote_footer() (#73)", {
  skip_on_cran()
  tex <- local_render_natbib_tex("tufte::tufte_handout")
  tex_joined <- paste(tex, collapse = "\n")
  expect_match(tex_joined, "\\\\hfill")
  expect_match(tex_joined, "\\\\citep\\[3\\]\\{dorian10\\}")
})

test_that("beamer_presentation renders natbib citation inside quote_footer() (#73)", {
  skip_on_cran()
  tex <- local_render_natbib_tex("beamer_presentation")
  tex_joined <- paste(tex, collapse = "\n")
  expect_match(tex_joined, "\\\\hfill")
  expect_match(tex_joined, "\\\\citep\\[3\\]\\{dorian10\\}")
})

test_that("tufte_handout PDF log contains no xcolor usenames warning (#127)", {
  skip_on_cran()
  rmd <- local_rmd_file(
    "---",
    "output: tufte::tufte_handout",
    "---",
    "",
    "Hello world.",
    .env = parent.frame()
  )
  output <- local_render_pdf(rmd)
  log_file <- xfun::with_ext(output, "log")
  skip_if(!file.exists(log_file), "LaTeX log not found")
  log_lines <- xfun::read_utf8(log_file)
  expect_false(
    any(grepl("usenames.*obsolete", log_lines, ignore.case = TRUE)),
    label = "xcolor 'usenames' obsolete warning found in LaTeX log"
  )
})

# natbib \citep / \citet routed to margin (#48) ----------------------------

test_that("patches/tufte-common.def redefines natbib \\citep and \\citet (#48)", {
  patches_dir <- pkg_file("rmarkdown", "templates", "tufte_handout", "patches")
  def_content <- xfun::read_utf8(file.path(patches_dir, "tufte-common.def"))
  # natbib emits \citep / \citet for [@key] / @key citations, but tufte-latex
  # only routes \cite through the margin sidenote machinery. The patch must
  # redefine \citep and \citet so pandoc-generated citations land in the
  # margin too.
  expect_true(
    any(grepl("RenewDocumentCommand.*\\\\citep", def_content)),
    label = "RenewDocumentCommand for \\citep present"
  )
  expect_true(
    any(grepl("RenewDocumentCommand.*\\\\citet", def_content)),
    label = "RenewDocumentCommand for \\citet present"
  )
  # The xparse signature must consume the optional star so raw-LaTeX
  # \citep*{key} / \citet*{key} routes through the same margin path
  # instead of mis-parsing the `*` as the citation key.
  expect_true(
    any(grepl("RenewDocumentCommand\\{\\\\citep\\}\\{s ", def_content)),
    label = "\\citep signature consumes star (s o o m)"
  )
  expect_true(
    any(grepl("RenewDocumentCommand\\{\\\\citet\\}\\{s ", def_content)),
    label = "\\citet signature consumes star (s o o m)"
  )
})

test_that("natbib citations render through tufte_handout (#48)", {
  skip_on_cran()
  rmd <- local_rmd_file(
    "---",
    "output:",
    "  tufte::tufte_handout:",
    "    citation_package: natbib",
    "bibliography: refs.bib",
    "---",
    "",
    "Parenthetical [@smith2000]. Textual @jones2010 said X.",
    .env = parent.frame()
  )
  writeLines(
    c(
      "@article{smith2000,author={Smith, John},title={t},journal={j},year={2000}}",
      "@book{jones2010,author={Jones, Alice},title={t},publisher={p},year={2010}}"
    ),
    file.path(dirname(rmd), "refs.bib")
  )
  output <- local_render_pdf(rmd)
  expect_true(file.exists(output))
})

test_that("raw-LaTeX \\citep* / \\citet* render through tufte_handout (#48)", {
  skip_on_cran()
  rmd <- local_rmd_file(
    "---",
    "output:",
    "  tufte::tufte_handout:",
    "    citation_package: natbib",
    "bibliography: refs.bib",
    "---",
    "",
    "Starred parenthetical \\citep*{smith2000}.",
    "",
    "Starred textual \\citet*{jones2010} said X.",
    .env = parent.frame()
  )
  writeLines(
    c(
      "@article{smith2000,author={Smith, John},title={t},journal={j},year={2000}}",
      "@book{jones2010,author={Jones, Alice},title={t},publisher={p},year={2010}}"
    ),
    file.path(dirname(rmd), "refs.bib")
  )
  output <- local_render_pdf(rmd)
  expect_true(file.exists(output))
})

#' @details `tufte_html()` provides the HTML format based on the Tufte CSS:
#'   <https://edwardtufte.github.io/tufte-css/>.
#' @param tufte_features A character vector of style features to enable:
#'   `fonts` stands for the `et-book` fonts in the `tufte-css`
#'   project, `background` means the lightyellow background color of the
#'   page, and `italics` means whether to use italics for the headers. You
#'   can enable a subset of these features, or just disable all of them by
#'   `NULL`. When this argument is not used and the `tufte_variant`
#'   argument is not `default`, no features are enabled.
#' @param tufte_variant A variant of the Tufte style. Currently supported styles
#'   are `default` (from the `tufte-css` project), and
#'   `envisioned` (inspired by the project `Envisioned CSS`
#'   <https://github.com/nogginfuel/envisioned-css> but essentially just
#'   sets the font family to `Roboto Condensed`, and changed the
#'   background/foreground colors).
#' @param margin_references Whether to place citations in margin notes.
#' @details `tufte_html2()` is a wrapper around [bookdown::html_document2()]
#'   that uses `tufte_html()` as the base format, enabling text references
#'   and cross-references. Requires the \pkg{bookdown} package.
#' @rdname tufte_handout
#' @export
tufte_html2 <- function(...) {
  check_bookdown()
  bookdown::html_document2(..., base_format = tufte_html)
}

#' @rdname tufte_handout
#' @export
tufte_html <- function(
  ...,
  tufte_features = c("fonts", "background", "italics"),
  tufte_variant = c("default", "envisioned"),
  margin_references = TRUE
) {
  tufte_variant <- match.arg(tufte_variant)
  if (missing(tufte_features) && tufte_variant != "default") {
    tufte_features <- character()
  }
  html_document2 <- function(..., extra_dependencies = list()) {
    rmarkdown::html_document(
      ...,
      extra_dependencies = c(
        extra_dependencies,
        tufte_html_dependency(tufte_features, tufte_variant)
      )
    )
  }
  format <- html_document2(theme = NULL, ...)
  pandoc2 <- pandoc2.0()

  # add --wrap=preserve to pandoc args for pandoc 2.0:
  format$pandoc$args <- add_wrap_preserve(format$pandoc$args, pandoc2)

  # when fig.margin = TRUE, set fig.beforecode = TRUE so plots are moved before
  # code blocks, and they can be top-aligned
  ohooks <- knitr::opts_hooks$set(fig.margin = function(options) {
    if (isTRUE(options$fig.margin)) {
      options$fig.beforecode <- TRUE
    }
    options
  })

  # make sure the existing post processor is called first in our new processor
  post_processor <- format$post_processor
  format$post_processor <- function(metadata, input, output, clean, verbose) {
    if (is.function(post_processor)) {
      output <- post_processor(metadata, input, output, clean, verbose)
    }

    knitr::opts_hooks$restore(ohooks)

    x <- xfun::read_utf8(output)
    fn_label <- paste0(knitr::opts_knit$get("rmarkdown.pandoc.id_prefix"), "fn")
    footnotes <- parse_footnotes(x, fn_label)
    notes <- footnotes$items
    # replace footnotes with sidenotes
    for (i in seq_along(notes)) {
      num <- sprintf(
        '<a href="#%s%d" class="%s" id="%sref%d"><sup>%d</sup></a>',
        fn_label,
        i,
        if (pandoc2) "footnote-ref" else "footnoteRef",
        fn_label,
        i,
        i
      )
      con <- sprintf(
        paste0(
          '<label for="tufte-sn-%d" class="margin-toggle sidenote-number">%d</label>',
          '<input type="checkbox" id="tufte-sn-%d" class="margin-toggle">',
          '<span class="sidenote"><span class="sidenote-number">%d</span> %s</span>'
        ),
        i,
        i,
        i,
        i,
        notes[i]
      )
      x <- gsub_fixed(num, con, x)
    }
    # remove footnotes at the bottom
    if (length(footnotes$range)) {
      x <- x[-footnotes$range]
    }

    # replace citations with margin notes
    if (margin_references) {
      x <- margin_references(x)
    }

    # place figure captions in margin notes
    x[x == '<p class="caption">'] <- '<p class="caption marginnote shownote">'

    # move </caption> to the same line as <caption>; the previous line should
    # start with <table
    for (i in intersect(grep("^<caption>", x), grep("^<table", x) + 1)) {
      j <- 0
      while (!grepl("</caption>$", x[i])) {
        j <- j + 1
        x[i] <- paste0(x[i], x[i + j])
        x[i + j] <- ""
      }
    }
    # place table captions in the margin
    r <- "^<caption>(.+)</caption>$"
    for (i in grep(r, x)) {
      # the previous line should be <table> or <table class=...>
      if (!grepl("^<table( class=.+)?>$", x[i - 1])) {
        next
      }
      cap <- gsub(r, "\\1", x[i])
      x[i] <- x[i - 1]
      x[i - 1] <- paste0(
        "<p><!--\n<caption>-->",
        '<span class="marginnote shownote">',
        cap,
        "</span><!--</caption>--></p>"
      )
    }

    # add an incremental number to the id of <label> and <input> for margin notes
    r <- '(<label|<input type="checkbox") (id|for)(="tufte-mn)-(" )'
    m <- gregexpr(r, x)
    j <- 1
    regmatches(x, m) <- lapply(regmatches(x, m), function(z) {
      n <- length(z)
      if (n == 0) {
        return(z)
      }
      if (n %% 2 != 0) {
        warning("The number of labels is different with checkboxes")
      }
      for (i in seq(1, n, 2)) {
        if (i + 1 > n) {
          break
        }
        z[i + (0:1)] <- gsub(r, paste0("\\1 \\2\\3-", j, "\\4"), z[i + (0:1)])
        j <<- j + 1
      }
      z
    })

    xfun::write_utf8(x, output)
    output
  }

  if (is.null(format$knitr$knit_hooks)) {
    format$knitr$knit_hooks <- list()
  }
  format$knitr$knit_hooks$plot <- function(x, options) {
    # make sure the plot hook always generates HTML code instead of ![]()
    if (is.null(options$out.extra)) {
      options$out.extra <- ""
    }
    fig_margin <- isTRUE(options$fig.margin)
    fig_fullwd <- isTRUE(options$fig.fullwidth)
    if (fig_margin || fig_fullwd) {
      if (is.null(options$fig.cap)) options$fig.cap <- " " # empty caption
    } else if (is.null(options$fig.topcaption)) {
      # for normal figures, place captions at the top of images
      options$fig.topcaption <- TRUE
    }
    res <- knitr::hook_plot_md(x, options)
    if (fig_margin) {
      res <- gsub_fixed(
        '<p class="caption">',
        '<!--\n<p class="caption marginnote">-->',
        res
      )
      res <- gsub_fixed("</p>", "<!--</p>-->", res)
      res <- gsub_fixed("</div>", "<!--</div>--></span></p>", res)
      # Use regex to match <div class="figure"> with optional style attribute
      # (e.g. from fig.align); the style is dropped since margin positioning
      # is handled by the marginnote CSS class
      res <- gsub(
        '<div class="figure"[^>]*>',
        paste0(
          "<p>",
          '<span class="marginnote shownote">',
          '\n<!--\n<div class="figure">-->'
        ),
        res
      )
    } else if (fig_fullwd) {
      # Use regex to match optional style attribute from fig.align
      res <- gsub(
        '<div class="figure"[^>]*>',
        '<div class="figure fullwidth">',
        res
      )
      res <- gsub_fixed(
        '<p class="caption">',
        '<p class="caption marginnote shownote">',
        res
      )
    }
    res
  }

  knitr::knit_engines$set(marginfigure = function(options) {
    options$type <- "marginnote"
    if (is.null(options$html.tag)) {
      options$html.tag <- "span"
    }
    options$html.before <- marginnote_html()
    eng_block <- knitr::knit_engines$get("block")
    eng_block(options)
  })

  # Expose the active tufte format to inline R helpers (e.g. quote_footer())
  # via knitr's opts_knit channel, so they can branch on tufte-vs-non-tufte
  # HTML output without relying on heuristics.
  if (is.null(format$knitr$opts_knit)) {
    format$knitr$opts_knit <- list()
  }
  format$knitr$opts_knit$tufte.format <- "html"

  format$inherits <- "html_document"

  format
}

#' @importFrom htmltools htmlDependency
tufte_html_dependency <- function(features, variant) {
  list(htmlDependency(
    "tufte-css",
    "2015.12.29",
    src = template_resources("tufte_html"),
    stylesheet = c(
      sprintf("tufte-%s.css", features),
      "tufte.css",
      if (variant != "default") sprintf("%s.css", variant)
    )
  ))
}

# we assume one footnote only contains one paragraph here, although it is
# possible to write multiple paragraphs in a footnote with Pandoc's Markdown
parse_footnotes <- function(x, fn_label = "fn") {
  i <- grep('^<div class="footnotes[^"]*"[^>]*>', x)
  if (length(i) == 0) {
    return(list(items = character(), range = integer()))
  }
  j <- which(x == "</div>")
  j <- min(j[j > i])
  n <- length(x)
  r <- sprintf(
    '<li id="%s([0-9]+)"><p>(.+)<a href="#%sref\\1"([^>]*)>.{1,2}</a></p></li>',
    fn_label,
    fn_label
  )
  s <- paste(x[i:j], collapse = "\n")
  list(
    items = gsub(r, "\\2", unlist(regmatches(s, gregexpr(r, s)))),
    range = i:j
  )
}

# move reference items from the bottom to the margin (as margin notes)
margin_references <- function(x) {
  i <- grep('^<div id="refs" class="references[^"]*"[^>]*>$', x)
  if (length(i) != 1) {
    return(x)
  }
  # link-citations: no
  if (length(grep('<a href="#ref-[^"]+"[^>]*>([^<]+)</a>', x)) == 0) {
    return(x)
  }
  r <- '^<div id="(ref-[^"]+)"[^>]*>$'
  k <- grep(r, x)
  k <- k[k > i]
  n <- length(k)
  if (n == 0) {
    return(x)
  }
  # pandoc-citeproc may generate a link on both the year and the alphabetic
  # suffix, e.g. <a href="#cite-key">2016</a><a href="#cite-key">a</a>;
  # Pandoc 2.11+ may also generate several links with parenthesis between links,
  # but this is different, e.g
  # <a href="#ref-R-rmarkdown" role="doc-biblioref">Allaire et al.</a> (<a href="#ref-R-rmarkdown" role="doc-biblioref">2020</a>)</span>
  # we need to merge the two links
  r2 <- '(<a href="#ref-[^"]+"[^>]*>)([^<]+)</a>([^<]*)\\1([^<]+)</a>([^<]*)'
  x <- gsub(r2, "\\1\\2\\3\\4\\5</a>", x)
  ids <- gsub(r, "\\1", x[k])
  ids <- sprintf('<a href="#%s"[^>]*>([^<]+)</a>', ids)
  ref <- gsub("^<p>|</p>$", "", x[k + 1])
  # replace 3 em-dashes with author names
  dashes <- paste0("^", intToUtf8(rep(8212, 3)), "[.]")
  for (j in grep(dashes, ref)) {
    ref[j] <- sub(dashes, sub("^([^.]+[.])( .+)$", "\\1", ref[j - 1]), ref[j])
  }
  ref <- marginnote_html(paste0('\\1<span class="marginnote">', ref, "</span>"))
  # Track which references have in-text citations (nocite entries won't match)
  matched <- logical(n)
  for (j in seq_len(n)) {
    if (any(grepl(ids[j], x))) {
      x <- gsub(ids[j], ref[j], x)
      matched[j] <- TRUE
    }
  }
  if (all(matched)) {
    x[-(i:(max(k) + 3))] # remove entire references section
  } else {
    # Remove only matched entries; keep nocite references at the bottom
    remove <- unlist(lapply(k[matched], function(pos) pos:(pos + 2)))
    if (length(remove) > 0) {
      x <- x[-remove]
    }
    x
  }
}

marginnote_html <- function(text = "", icon = "&#8853;") {
  sprintf(
    paste0(
      '<label for="tufte-mn-" class="margin-toggle">%s</label>',
      '<input type="checkbox" id="tufte-mn-" class="margin-toggle">%s'
    ),
    icon,
    text
  )
}

#' Tufte handout formats (PDF and HTML)
#'
#' Templates for creating handouts according to the style of Edward R. Tufte and
#' Richard Feynman.
#'
#' `tufte_handout()` provides the PDF format based on the Tufte-LaTeX
#' class: <https://tufte-latex.github.io/tufte-latex/>.
#' @inheritParams rmarkdown::pdf_document
#' @param margin_fig_pos Default vertical offset for margin figures (a LaTeX
#'   length such as `"0cm"` or `"-5pt"`). When set, this value is used as the
#'   `fig.pos` for all chunks with `fig.margin = TRUE`, unless the chunk
#'   specifies its own `fig.pos`. This avoids the problem of setting `fig.pos`
#'   globally via `knitr::opts_chunk$set()`, which would also affect regular
#'   figures where `fig.pos` is a placement specifier (e.g. `"htbp"`), not a
#'   length. Can also be set per-chunk or via `knitr::opts_chunk$set()` as the
#'   `margin_fig_pos` chunk option. Defaults to `NULL` (use the Tufte-LaTeX
#'   class default of `-1.2ex`).
#' @param ... Other arguments to be passed to [rmarkdown::pdf_document()] or
#'   [rmarkdown::html_document()] (note you cannot use the `template`
#'   argument in `tufte_handout` or the `theme` argument in
#'   `tufte_html()`; these arguments have been set internally)
#' @return `tufte_handout()`, `tufte_book()`, `tufte_html()`, and the
#'   bookdown wrappers `tufte_handout2()`, `tufte_book2()`, and
#'   `tufte_html2()` return an R Markdown output format object that can
#'   be passed to [rmarkdown::render()]. The inline helpers
#'   `newthought()`, `margin_note()`, `quote_footer()`, and
#'   `sans_serif()` return a character string with HTML or LaTeX markup
#'   depending on the active output format.
#' @references See <https://rstudio.github.io/tufte/> for an example.
#' @export
#' @examples library(tufte)
tufte_handout <- function(
  fig_width = 4,
  fig_height = 2.5,
  fig_crop = "auto",
  dev = "pdf",
  highlight = "default",
  margin_fig_pos = NULL,
  ...
) {
  tufte_pdf(
    "tufte-handout",
    fig_width,
    fig_height,
    fig_crop,
    dev,
    highlight,
    margin_fig_pos = margin_fig_pos,
    ...
  )
}

#' @rdname tufte_handout
#' @export
tufte_book <- function(
  fig_width = 4,
  fig_height = 2.5,
  fig_crop = "auto",
  dev = "pdf",
  highlight = "default",
  margin_fig_pos = NULL,
  ...
) {
  tufte_pdf(
    "tufte-book",
    fig_width,
    fig_height,
    fig_crop,
    dev,
    highlight,
    margin_fig_pos = margin_fig_pos,
    ...
  )
}

#' @details `tufte_handout2()` and `tufte_book2()` are wrappers around
#'   [bookdown::pdf_book()] that use the corresponding tufte format as the
#'   base format. They support bookdown text references (`(ref:label)`) in
#'   figure and table captions, cross-references, and other bookdown
#'   extensions. Requires the \pkg{bookdown} package to be installed.
#' @rdname tufte_handout
#' @export
tufte_handout2 <- function(...) {
  check_bookdown()
  bookdown::pdf_book(..., base_format = tufte_handout)
}

#' @rdname tufte_handout
#' @export
tufte_book2 <- function(...) {
  check_bookdown()
  bookdown::pdf_book(..., base_format = tufte_book)
}

tufte_pdf <- function(
  documentclass = c("tufte-handout", "tufte-book"),
  fig_width = 4,
  fig_height = 2.5,
  fig_crop = "auto",
  dev = "pdf",
  highlight = "default",
  margin_fig_pos = NULL,
  template = template_resources("tufte_handout", "tufte-handout.tex"),
  ...
) {
  # resolve default highlight
  if (identical(highlight, "default")) {
    highlight <- "pygments"
  }

  # call the base pdf_document format with the appropriate options
  format <- rmarkdown::pdf_document(
    fig_width = fig_width,
    fig_height = fig_height,
    fig_crop = fig_crop,
    dev = dev,
    highlight = highlight,
    template = template,
    ...
  )

  # LaTeX document class
  documentclass <- match.arg(documentclass)
  format$pandoc$args <- c(
    format$pandoc$args,
    "--variable",
    paste0("documentclass:", documentclass),
    if (documentclass == "tufte-book") {
      if (pandoc2.0()) "--top-level-division=chapter" else "--chapters"
    }
  )

  # Prepend our patched tufte-common.def to TEXINPUTS so kpathsea finds it
  # before the system version. Fixes the xcolor usenames warning (#127).
  patches_dir <- pkg_file("rmarkdown", "templates", "tufte_handout", "patches")
  old_texinputs <- NULL
  base_pre_processor <- format$pre_processor
  format$pre_processor <- function(metadata, input_file, ...) {
    old_texinputs <<- Sys.getenv("TEXINPUTS", unset = "")
    # Trailing path separator tells kpathsea to also search default paths
    Sys.setenv(
      TEXINPUTS = paste0(patches_dir, .Platform$path.sep, old_texinputs)
    )
    if (is.function(base_pre_processor)) {
      base_pre_processor(metadata, input_file, ...)
    } else {
      character(0)
    }
  }
  # on_exit runs even when render() errors, so TEXINPUTS is always restored.
  format$on_exit <- function() {
    if (!is.null(old_texinputs)) {
      Sys.setenv(TEXINPUTS = old_texinputs)
    }
  }

  knitr::knit_engines$set(marginfigure = function(options) {
    options$type <- "marginfigure"
    eng_block <- knitr::knit_engines$get("block")
    eng_block(options)
  })

  # create knitr options (ensure opts and hooks are non-null)
  knitr_options <- rmarkdown::knitr_options_pdf(
    fig_width,
    fig_height,
    fig_crop,
    dev
  )
  if (is.null(knitr_options$opts_knit)) {
    knitr_options$opts_knit <- list()
  }
  if (is.null(knitr_options$knit_hooks)) {
    knitr_options$knit_hooks <- list()
  }

  # set options
  knitr_options$opts_knit$width <- 45
  # Expose the active tufte format to inline R helpers (e.g. quote_footer())
  # via knitr's opts_knit channel.
  knitr_options$opts_knit$tufte.format <- "handout"
  if (!is.null(margin_fig_pos)) {
    knitr_options$opts_chunk$margin_fig_pos <- margin_fig_pos
  }

  # set hooks for special plot output
  knitr_options$knit_hooks$plot <- function(x, options) {
    # determine figure type
    if (isTRUE(options$fig.margin)) {
      options$fig.env <- "marginfigure"
      if (is.null(options$fig.cap)) {
        options$fig.cap <- ""
      }
      # Apply margin_fig_pos as the vertical offset for margin figures,
      # unless this chunk has its own fig.pos (#62). `options$fig.pos` is
      # the merged value, so checking it directly would also fire for a
      # global `knitr::opts_chunk$set(fig.pos = "htbp")` intended for
      # regular figures, yielding `\begin{marginfigure}[htbp]` (invalid:
      # the optional argument is a length, not a placement specifier).
      # chunk_overrides_fig_pos() distinguishes chunk-level values (header
      # literal or opts_template/opts.label injection) from the merged
      # global default.
      if (
        !is.null(options$margin_fig_pos) &&
          !chunk_overrides_fig_pos(options)
      ) {
        options$fig.pos <- options$margin_fig_pos
      }
    } else if (isTRUE(options$fig.fullwidth)) {
      options$fig.env <- "figure*"
      if (is.null(options$fig.cap)) options$fig.cap <- ""
    }

    knitr::hook_plot_tex(x, options)
  }

  # override the knitr settings of the base format and return the format
  format$knitr <- knitr_options
  format$inherits <- "pdf_document"
  format
}

#!/bin/bash

cd inst/rmarkdown/templates/tufte_html/resources/
cp ../../../../../../tufte-css/{tufte.css,LICENSE} ./
[ ! -d et-book ] && mkdir et-book
find ../../../../../../tufte-css/et-book -type f -name \*.ttf -exec cp {} et-book/ \;
cd et-book
for i in `ls *.ttf`; do
  mv $i ${i#et-book-}
done
sed -e 's@et-book/et-book-.*/@@' -e 's@\(url."et-book\)-@\1/@' -i '' ../tufte.css

---
title: "Tufte样式"
subtitle: "一个R Markdown实现"
author: "JJ Allaire，谢益辉"
date: "`r Sys.Date()`"
output:
  tufte::tufte_html: default
  tufte::tufte_book:
    citation_package: natbib
    latex_engine: xelatex
  tufte::tufte_handout:
    citation_package: natbib
    latex_engine: xelatex
ctex: yes
biblio-title: 参考文献
bibliography: skeleton.bib
link-citations: yes
---

```{r setup, include=FALSE}
library(tufte)
# tufte版本变化之后更新knitr缓存
knitr::opts_chunk$set(cache.extra = packageVersion('tufte'))
options(htmltools.dir.version = FALSE)
```

# 引言

Tufte样式在Edward Tufte的书以及物理学家费曼的教科书很常见，它的一个显著特点就是边栏的使用。例如脚注和边栏注解，以及放在边栏里的小型插图。Tufte样式在LaTeX和HTML/CSS中都有实现^[参见Github库[tufte-latex](https://github.com/tufte-latex/tufte-latex)和 [tufte-css](https://github.com/edwardtufte/tufte-css)]。我们将这两种实现都纳入了[**tufte**包](https://github.com/rstudio/tufte)。若需要LaTeX/PDF输出，使用输出格式`tufte_handout`即可，类似地，`tufte_book`可以用来输出PDF书，`tufte_html`生成HTML网页。这些输出格式可以在YAML元数据中指定，或者传给`rmarkdown::render()`函数。若对**rmarkdown**包不熟悉，可参见 @R-rmarkdown。

```yaml
---
title: "一个Tufte样式示例"
author: "张三"
ctex: yes
output:
  tufte::tufte_handout:
    latex_engine: xelatex
  tufte::tufte_html: default
---
```

# 章节标题

Tufte样式不主张太深的章节目录，一般仅仅使用一级标题（Markdown中用一个井号`#`）和二级标题（两个井号）。

# 插图

## 边栏插图

插图在Tufte的书中非常常见，我们可以使用三种插图：边栏图、全宽图、主栏图。先说边栏图：使用**knitr**代码段选项`fig.margin = TRUE`即可将图形放置在边栏中，如：

```{r fig-margin, fig.margin = TRUE, fig.cap = "MPG与horsepower两个变量的散点图；颜色代表自动挡或手动挡。", fig.width=3.5, fig.height=3.5, cache=TRUE, message=FALSE}
library(ggplot2)
mtcars2 <- mtcars
mtcars2$am <- factor(
  mtcars$am, labels = c('automatic', 'manual')
)
ggplot(mtcars2, aes(hp, mpg, color = am)) +
  geom_point() + geom_smooth() +
  theme(legend.position = 'bottom')
```

注意我们使用代码段选项`fig.cap`设定了图的标题。当然我们也可以设置图的长宽。

## 任意边栏内容

事实上我们可以在边栏中放置除了图之外的内容，此时我们不再使用```` ```{r} ````写代码段，而是用```` ```{marginfigure} ````。例如右边有一个微积分第一基本定理。

```{marginfigure}
根据微积分第一基本定理我们知道，对$x \in [a, b]$有
$$\frac{d}{dx}\left( \int_{a}^{x} f(u)\,du\right)=f(x).$$
```

为了文本内容的可移植性（同样的内容可以生成HTML和LaTeX文档），我们建议边栏中不要放置太复杂的内容，简单的加粗、倾斜都没有问题，但不建议在边栏中使用列表、参考文献等内容。

## 全宽插图

代码段选项`fig.fullwidth = TRUE`可以使得一幅图占用全部页宽，例如：

```{r fig-fullwidth, fig.width = 10, fig.height = 2, fig.fullwidth = TRUE, fig.cap = "一幅全宽图形。", warning=FALSE, cache=TRUE, message=FALSE}
ggplot(diamonds, aes(carat, price)) + geom_smooth() +
  facet_grid(~ cut)
```

其它和图形有关的代码段选项仍然可以使用，一般情况下，全宽图形的`fig.width`选项会较大，而`fig.height`相对较小。上图的尺寸是$10 \times 2$英寸.

## 主栏插图

默认情况下，R代码段生成的图形放置在主栏里，其标题放在边栏中，例如：

```{r fig-main, fig.cap = "一幅主栏插图。", cache=TRUE}
ggplot(diamonds, aes(cut, price)) + geom_boxplot()
```

# 边栏附注

Tufte样式的文档中，脚注会被自动转换为边栏附注^[这里本来是一个脚注]。脚注是带编号的，另一种边栏附注是不带编号的，这种附注需要用**tufte**包中的R函数`margin_note()`在**knitr**行内代码中生成。`r margin_note("这是一个边栏附注，它没有编号。")`与边栏插图一样，边栏附注中我们也不建议写太复杂的内容，通常只是一句简单的文字。

# 参考文献

HTML输出中，参考文献默认也放在边栏中。例如这里我们可以引用[@R-base]。这个功能需要在YAML元数据中设置`link-citations: yes`，而且`pandoc-citeproc`程序的版本应该至少是0.7.2。若这两个条件不满足，参考文献会被放在文档末尾。

# 表格

我们可以用**knitr**包中的`kable()`函数生成简单的表格。HTML输出中表格的标题也会被放在边栏中。

```{r}
knitr::kable(
  mtcars[1:6, 1:6], caption = 'mtcars数据的前几行。'
)
```

# 引文

Markdown语法使用`>`来生成引文，如果需要在引文下面用行内代码以及`quote_footer()`函数加上一句引文来源，例如：

> "多亏了我的律师，要不然我现在还在牢里。两个人一起挖确实比一个人快很多啊。"
>
> `r tufte::quote_footer('--- Joe Martin')`

如果不用这个函数的话，引文底部的话只是一个普通段落：

> "伟人论道，凡人论事，小人论酒。"
>
> --- Fran Lebowitz

# 响应式页面

这个包生成的HTML页面是响应式的：如果页宽小于760像素，边栏内容会自动隐藏。此时我们可以点击脚注的序号显示它，其它边栏附注则可以通过点击圆圈加号的符号显示。

# 结语

希望诸位喜欢R Markdown的超级简洁性，同时我们感谢Tufte-CSS和Tufte-LaTeX项目的作者们，没有他们的辛勤劳动，就没有这个**tufte**包。这份文档的R Markdown源文档可以在[Github上找到](https://github.com/rstudio/tufte/raw/main/inst/rmarkdown/templates/tufte_ctex/skeleton/skeleton.Rmd)，或者直接使用RStudio菜单`File -> New File -> R Markdown -> From Template`新建一个文档，或直接从R里面打开这个Rmd文件：

```{r eval=FALSE}
file.edit(
  tufte:::template_resources(
    'tufte_ctex', '..', 'skeleton', 'skeleton.Rmd'
  )
)
```

```{r bib, include=FALSE}
# create a bib file for the R packages used in this document
knitr::write_bib(c('base', 'rmarkdown'), file = 'skeleton.bib')
```

---
title: "A Tufte Handout Example"
runningheader: "Tufte Handout with R Markdown" # only for pdf output
subtitle: "An implementation in R Markdown" # only for html output
author: "JJ Allaire and Yihui Xie"
date: "`r Sys.Date()`"
output:
  tufte::tufte_html: default
  tufte::tufte_handout:
    citation_package: natbib
    latex_engine: xelatex
  tufte::tufte_book:
    citation_package: natbib
    latex_engine: xelatex
bibliography: skeleton.bib
link-citations: yes
---

```{r setup, include=FALSE}
library(tufte)
# invalidate cache when the tufte version changes
knitr::opts_chunk$set(cache.extra = packageVersion('tufte'))
options(htmltools.dir.version = FALSE)
```

# Introduction

The Tufte handout style is a style that Edward Tufte uses in his books and handouts. Tufte's style is known for its extensive use of sidenotes, tight integration of graphics with text, and well-set typography. This style has been implemented in LaTeX and HTML/CSS^[See Github repositories [tufte-latex](https://github.com/tufte-latex/tufte-latex) and [tufte-css](https://github.com/edwardtufte/tufte-css)], respectively. We have ported both implementations into the [**tufte** package](https://github.com/rstudio/tufte). If you want LaTeX/PDF output, you may use the `tufte_handout` format for handouts, and `tufte_book` for books. For HTML output, use `tufte_html`. These formats can be either specified in the YAML metadata at the beginning of an R Markdown document (see an example below), or passed to the `rmarkdown::render()` function. See @R-rmarkdown for more information about **rmarkdown**.

```yaml
---
title: "An Example Using the Tufte Style"
author: "John Smith"
output:
  tufte::tufte_handout: default
  tufte::tufte_html: default
---
```

There are two goals of this package:

1. To produce both PDF and HTML output with similar styles from the same R Markdown document;
1. To provide simple syntax to write elements of the Tufte style such as side notes and margin figures, e.g. when you want a margin figure, all you need to do is the chunk option `fig.margin = TRUE`, and we will take care of the details for you, so you never need to think about `\begin{marginfigure} \end{marginfigure}` or `<span class="marginfigure"> </span>`; the LaTeX and HTML code under the hood may be complicated, but you never need to learn or write such code.

If you have any feature requests or find bugs in **tufte**, please do not hesitate to file them to https://github.com/rstudio/tufte/issues. For general questions, you may ask them on StackOverflow: https://stackoverflow.com/tags/rmarkdown.

# Headings

This style provides first and second-level headings (that is, `#` and `##`), demonstrated in the next section. You may get unexpected output if you try to use `###` and smaller headings.

`r newthought('In his later books')`^[[Beautiful Evidence](https://www.edwardtufte.com/tufte/books_be)], Tufte starts each section with a bit of vertical space, a non-indented paragraph, and sets the first few words of the sentence in small caps. To accomplish this using this style, call the `newthought()` function in **tufte** in an _inline R expression_ `` `r ` `` as demonstrated at the beginning of this paragraph.^[Note you should not assume **tufte** has been attached to your R session. You should either `library(tufte)` in your R Markdown document before you call `newthought()`, or use `tufte::newthought()`.]

# Figures

## Margin Figures

Images and graphics play an integral role in Tufte's work. To place figures in the margin you can use the **knitr** chunk option `fig.margin = TRUE`. For example:

```{r fig-margin, fig.margin = TRUE, fig.cap = "MPG vs horsepower, colored by transmission.", fig.width=3.5, fig.height=3.5, cache=TRUE, message=FALSE}
library(ggplot2)
mtcars2 <- mtcars
mtcars2$am <- factor(
  mtcars$am, labels = c('automatic', 'manual')
)
ggplot(mtcars2, aes(hp, mpg, color = am)) +
  geom_point() + geom_smooth() +
  theme(legend.position = 'bottom')
```

Note the use of the `fig.cap` chunk option to provide a figure caption. You can adjust the proportions of figures using the `fig.width` and `fig.height` chunk options. These are specified in inches, and will be automatically scaled down to fit within the handout margin.

## Arbitrary Margin Content

In fact, you can include anything in the margin using the **knitr** engine named `marginfigure`. Unlike R code chunks ```` ```{r} ````, you write a chunk starting with ```` ```{marginfigure} ```` instead, then put the content in the chunk. See an example on the right about the first fundamental theorem of calculus.

```{marginfigure}
We know from _the first fundamental theorem of calculus_ that for $x$ in $[a, b]$:
$$\frac{d}{dx}\left( \int_{a}^{x} f(u)\,du\right)=f(x).$$
```

For the sake of portability between LaTeX and HTML, you should keep the margin content as simple as possible (syntax-wise) in the `marginefigure` blocks. You may use simple Markdown syntax like `**bold**` and `_italic_` text, but please refrain from using footnotes, citations, or block-level elements (e.g. blockquotes and lists) there.

Note: if you set `echo = FALSE` in your global chunk options, you will have to add `echo = TRUE` to the chunk to display a margin figure, for example ```` ```{marginfigure, echo = TRUE} ````.

## Full Width Figures

You can arrange for figures to span across the entire page by using the chunk option `fig.fullwidth = TRUE`.

```{r fig-fullwidth, fig.width = 10, fig.height = 2, fig.fullwidth = TRUE, fig.cap = "A full width figure.", warning=FALSE, message=FALSE, cache=TRUE}
ggplot(diamonds, aes(carat, price)) + geom_smooth() +
  facet_grid(~ cut)
```

Other chunk options related to figures can still be used, such as `fig.width`, `fig.cap`, `out.width`, and so on. For full width figures, usually `fig.width` is large and `fig.height` is small. In the above example, the plot size is $10 \times 2$.

## Arbitrary Full Width Content

Any content can span to the full width of the page. This feature requires Pandoc 2.0 or above. All you need is to put your content in a fenced `Div` with the class `fullwidth`, e.g.,

```md
::: {.fullwidth}
Any _full width_ content here.
:::
```

Below is an example:

::: {.fullwidth}
_R is free software and comes with ABSOLUTELY NO WARRANTY._ You are welcome to redistribute it under the terms of the GNU General Public License versions 2 or 3. For more information about these matters see https://www.gnu.org/licenses/.
:::

## Main Column Figures

Besides margin and full width figures, you can of course also include figures constrained to the main column. This is the default type of figures in the LaTeX/HTML output.

```{r fig-main, fig.cap = "A figure in the main column.", cache=TRUE}
ggplot(diamonds, aes(cut, price)) + geom_boxplot()
```

# Sidenotes

One of the most prominent and distinctive features of this style is the extensive use of sidenotes. There is a wide margin to provide ample room for sidenotes and small figures. Any use of a footnote will automatically be converted to a sidenote. ^[This is a sidenote that was entered using a footnote.] 

If you'd like to place ancillary information in the margin without the sidenote mark (the superscript number), you can use the `margin_note()` function from **tufte** in an inline R expression. `r margin_note("This is a margin note.  Notice that there is no number preceding the note.")` This function does not process the text with Pandoc, so Markdown syntax will not work here. If you need to write anything in Markdown syntax, please use the `marginfigure` block described previously.

# References

References can be displayed as margin notes for HTML output. For example, we can cite R here [@R-base]. To enable this feature, you must set `link-citations: yes` in the YAML metadata, and the version of Pandoc should at least 2.11 or `pandoc-citeproc` should be available and at least 0.7.2. You can always install your own version of Pandoc from https://pandoc.org/installing.html if the version is not sufficient. To check the version of `pandoc-citeproc` in your system, you may run this in R:

```{r eval=FALSE}
system2('pandoc-citeproc', '--version')
```

If your version of `pandoc-citeproc` is too low, or you did not set `link-citations: yes` in YAML, references in the HTML output will be placed at the end of the output document.

# Tables

You can use the `kable()` function from the **knitr** package to format tables that integrate well with the rest of the Tufte handout style. The table captions are placed in the margin like figures in the HTML output.

```{r}
knitr::kable(
  mtcars[1:6, 1:6], caption = 'A subset of mtcars.'
)
```

# Block Quotes

We know from the Markdown syntax that paragraphs that start with `>` are converted to block quotes. If you want to add a right-aligned footer for the quote, you may use the function `quote_footer()` from **tufte** in an inline R expression. Here is an example:

> "If it weren't for my lawyer, I'd still be in prison. It went a lot faster with two people digging."
>
> `r tufte::quote_footer('--- Joe Martin')`

Without using `quote_footer()`, it looks like this (the second line is just a normal paragraph):

> "Great people talk about ideas, average people talk about things, and small people talk about wine."
>
> --- Fran Lebowitz

# Responsiveness

The HTML page is responsive in the sense that when the page width is smaller than 760px, sidenotes and margin notes will be hidden by default. For sidenotes, you can click their numbers (the superscripts) to toggle their visibility. For margin notes, you may click the circled plus signs to toggle visibility.

# More Examples

The rest of this document consists of a few test cases to make sure everything still works well in slightly more complicated scenarios. First we generate two plots in one figure environment with the chunk option `fig.show = 'hold'`:

```{r fig-two-together, fig.cap="Two plots in one figure environment.", fig.show='hold', cache=TRUE, message=FALSE}
p <- ggplot(mtcars2, aes(hp, mpg, color = am)) +
  geom_point()
p
p + geom_smooth()
```

Then two plots in separate figure environments (the code is identical to the previous code chunk, but the chunk option is the default `fig.show = 'asis'` now):

```{r fig-two-separate, ref.label='fig-two-together', fig.cap=sprintf("Two plots in separate figure environments (the %s plot).", c("first", "second")), cache=TRUE, message=FALSE}
```

You may have noticed that the two figures have different captions, and that is because we used a character vector of length 2 for the chunk option `fig.cap` (something like `fig.cap = c('first plot', 'second plot')`).

Next we show multiple plots in margin figures. Similarly, two plots in the same figure environment in the margin:

```{r fig-margin-together, fig.margin=TRUE, fig.show='hold', fig.cap="Two plots in one figure environment in the margin.", fig.width=3.5, fig.height=2.5, cache=TRUE}
p
p + geom_smooth(method = 'lm')
```

Then two plots from the same code chunk placed in different figure environments:

```{r fig-margin-separate, fig.margin=TRUE, fig.cap=sprintf("Two plots in separate figure environments in the margin (the %s plot).", c("first", "second")), fig.width=3.5, fig.height=2.5, cache=TRUE}
knitr::kable(head(iris, 15))
p
knitr::kable(head(iris, 12))
p + geom_smooth(method = 'lm')
knitr::kable(head(iris, 5))
```

We blended some tables in the above code chunk only as _placeholders_ to make sure there is enough vertical space among the margin figures, otherwise they will be stacked tightly together. For a practical document, you should not insert too many margin figures consecutively and make the margin crowded. 

You do not have to assign captions to figures. We show three figures with no captions below in the margin, in the main column, and in full width, respectively.

```{r fig-nocap-margin, fig.margin=TRUE, fig.width=3.5, fig.height=2, cache=TRUE}
# a boxplot of weight vs transmission; this figure
# will be placed in the margin
ggplot(mtcars2, aes(am, wt)) + geom_boxplot() +
  coord_flip()
```
```{r fig-nocap-main, cache=TRUE}
# a figure in the main column
p <- ggplot(mtcars, aes(wt, hp)) + geom_point()
p
```
```{r fig-nocap-fullwidth, fig.fullwidth=TRUE, fig.width=10, fig.height=3, cache=TRUE}
# a fullwidth figure
p + geom_smooth(method = 'lm') + facet_grid(~ gear)
```

# Some Notes on Tufte CSS

There are a few other things in Tufte CSS that we have not mentioned so far. If you prefer `r sans_serif('sans-serif fonts')`, use the function `sans_serif()` in **tufte**. For epigraphs, you may use a pair of underscores to make the paragraph italic in a block quote, e.g.

> _I can win an argument on any topic, against any opponent. People know this, and steer clear of me at parties. Often, as a sign of their great respect, they don't even invite me._
>
> `r quote_footer('--- Dave Barry')`

We hope you will enjoy the simplicity of R Markdown and this R package, and we sincerely thank the authors of the Tufte-CSS and Tufte-LaTeX projects for developing the beautiful CSS and LaTeX classes. Our **tufte** package would not have been possible without their heavy lifting.

You can turn on/off some features of the Tufte style in HTML output. The default features enabled are:

```yaml
output:
  tufte::tufte_html:
    tufte_features: ["fonts", "background", "italics"]
```

If you do not want the page background to be lightyellow, you can remove `background` from `tufte_features`. You can also customize the style of the HTML page via a CSS file. For example, if you do not want the subtitle to be italic, you can define

```css
h3.subtitle em {
  font-style: normal;
}
```

in, say, a CSS file `my_style.css` (under the same directory of your Rmd document), and apply it to your HTML output via the `css` option, e.g.,

```yaml
output:
  tufte::tufte_html:
    tufte_features: ["fonts", "background"]
    css: "my_style.css"
```

There is also a variant of the Tufte style in HTML/CSS named "[Envisoned CSS](https://github.com/nogginfuel/envisioned-css)". This style can be used by specifying the argument `tufte_variant = 'envisioned'` in `tufte_html()`^[The actual Envisioned CSS was not used in the **tufte** package. We only changed the fonts, background color, and text color based on the default Tufte style.], e.g.

```yaml
output:
  tufte::tufte_html:
    tufte_variant: "envisioned"
```

To see the R Markdown source of this example document, you may follow [this link to Github](https://github.com/rstudio/tufte/raw/main/inst/rmarkdown/templates/tufte_html/skeleton/skeleton.Rmd), use the wizard in RStudio IDE (`File -> New File -> R Markdown -> From Template`), or open the Rmd file in the package:

```{r eval=FALSE}
file.edit(
  tufte:::template_resources(
    'tufte_html', '..', 'skeleton', 'skeleton.Rmd'
  )
)
```

This document is also available in [Chinese](https://rstudio.github.io/tufte/cn/), and its `envisioned` style can be found [here](https://rstudio.github.io/tufte/envisioned/).

```{r bib, include=FALSE}
# create a bib file for the R packages used in this document
knitr::write_bib(c('base', 'rmarkdown'), file = 'skeleton.bib')
```

---
title: "Test margin references"
link-citations: {{linked}}
references:
- author:
  - family: Xie
    given: Yihui
  id: R-knitr
  issued: 
    year: 2020
  note: R package version 1.30
  title: "Knitr: A general-purpose package for dynamic report generation in r"
  title-short: Knitr
  type: book
  url: "https://yihui.org/knitr/"
- author:
  - family: Xie
    given: Yihui
  id: R-knitr2
  issued: 
    year: 2020
  note: R package version 1.29
  title: "Knitr: A general-purpose package for dynamic report generation in r - duplicate"
  title-short: Knitr
  type: book
  url: "https://yihui.org/knitr/"
- author:
  - family: Xie
    given: Yihui
  - family: Allaire
    given: J. J.
  - family: Grolemund
    given: Garrett
  id: rmarkdown2018
  issued: 
    year: 2018
  note: ISBN 9781138359338
  publisher: Chapman; Hall/CRC
  publisher-place: Boca Raton, Florida
  title: "R markdown: The definitive guide"
  title-short: R markdown
  type: book
  url: "https://bookdown.org/yihui/rmarkdown"
output: tufte::tufte_html
---

See [@R-knitr]. 

See @R-knitr2

See @R-knitr and @rmarkdown2018

See [@R-knitr] and [@rmarkdown2018]

# put references in margin when link-citations: yes

    Code
      margin_references(x)
    Output
      [1] "<p>See <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie 2020a<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>.</span>)</span>.</p>"                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
      [2] "<p>See <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie (2020b)<span class=\"marginnote\">Xie, Yihui. 2020b. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r - Duplicate</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>.</span></span></p>"                                                                                                                                                                                                                                                                                                                                                                                                                                                                    
      [3] "<p>See <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie (2020a)<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>.</span></span> and <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie, Allaire, and Grolemund (2018)<span class=\"marginnote\">Xie, Yihui, J. J. Allaire, and Garrett Grolemund. 2018. <em>R Markdown: The Definitive Guide</em>. Boca Raton, Florida: Chapman; Hall/CRC. <a href=\"https://bookdown.org/yihui/rmarkdown\">https://bookdown.org/yihui/rmarkdown</a>.</span></span></p>"
      [4] "<p>See <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie 2020a<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>.</span>)</span> and <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie, Allaire, and Grolemund 2018<span class=\"marginnote\">Xie, Yihui, J. J. Allaire, and Garrett Grolemund. 2018. <em>R Markdown: The Definitive Guide</em>. Boca Raton, Florida: Chapman; Hall/CRC. <a href=\"https://bookdown.org/yihui/rmarkdown\">https://bookdown.org/yihui/rmarkdown</a>.</span>)</span></p>"

---

    Code
      margin_references(x)
    Output
       [1] "<p>See <span class=\"citation\">(Xie 2020a)</span>.</p>"                                                                                                                                                                              
       [2] "<p>See <span class=\"citation\">Xie (2020b)</span></p>"                                                                                                                                                                               
       [3] "<p>See <span class=\"citation\">Xie (2020a)</span> and <span class=\"citation\">Xie, Allaire, and Grolemund (2018)</span></p>"                                                                                                        
       [4] "<p>See <span class=\"citation\">(Xie 2020a)</span> and <span class=\"citation\">(Xie, Allaire, and Grolemund 2018)</span></p>"                                                                                                        
       [5] "<div id=\"refs\" class=\"references csl-bib-body hanging-indent\">"                                                                                                                                                                   
       [6] "<div id=\"ref-R-knitr\" class=\"csl-entry\">"                                                                                                                                                                                         
       [7] "Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>."                                                                  
       [8] "</div>"                                                                                                                                                                                                                               
       [9] "<div id=\"ref-R-knitr2\" class=\"csl-entry\">"                                                                                                                                                                                        
      [10] "———. 2020b. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r - Duplicate</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>."                                                             
      [11] "</div>"                                                                                                                                                                                                                               
      [12] "<div id=\"ref-rmarkdown2018\" class=\"csl-entry\">"                                                                                                                                                                                   
      [13] "Xie, Yihui, J. J. Allaire, and Garrett Grolemund. 2018. <em>R Markdown: The Definitive Guide</em>. Boca Raton, Florida: Chapman; Hall/CRC. <a href=\"https://bookdown.org/yihui/rmarkdown\">https://bookdown.org/yihui/rmarkdown</a>."
      [14] "</div>"                                                                                                                                                                                                                               
      [15] "</div>"                                                                                                                                                                                                                               

# put references in margin when link-citations: yes using csl

    Code
      margin_references(x)
    Output
      [1] "<p>See <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie, 2020a<span class=\"marginnote\">Xie, Y. (2020a). <em>Knitr: A general-purpose package for dynamic report generation in r</em>. Retrieved from <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a></span>)</span>.</p>"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
      [2] "<p>See <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie (2020b)<span class=\"marginnote\">Xie, Y. (2020b). <em>Knitr: A general-purpose package for dynamic report generation in r - duplicate</em>. Retrieved from <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a></span></span></p>"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   
      [3] "<p>See <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie (2020a)<span class=\"marginnote\">Xie, Y. (2020a). <em>Knitr: A general-purpose package for dynamic report generation in r</em>. Retrieved from <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a></span></span> and <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie, Allaire, &amp; Grolemund (2018)<span class=\"marginnote\">Xie, Y., Allaire, J. J., &amp; Grolemund, G. (2018). <em>R markdown: The definitive guide</em>. Boca Raton, Florida: Chapman; Hall/CRC. Retrieved from <a href=\"https://bookdown.org/yihui/rmarkdown\">https://bookdown.org/yihui/rmarkdown</a></span></span></p>"  
      [4] "<p>See <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie, 2020a<span class=\"marginnote\">Xie, Y. (2020a). <em>Knitr: A general-purpose package for dynamic report generation in r</em>. Retrieved from <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a></span>)</span> and <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie, Allaire, &amp; Grolemund, 2018<span class=\"marginnote\">Xie, Y., Allaire, J. J., &amp; Grolemund, G. (2018). <em>R markdown: The definitive guide</em>. Boca Raton, Florida: Chapman; Hall/CRC. Retrieved from <a href=\"https://bookdown.org/yihui/rmarkdown\">https://bookdown.org/yihui/rmarkdown</a></span>)</span></p>"

---

    Code
      margin_references(x)
    Output
      [1] "<p>See <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie 2020a<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>.</span>)</span>.</p>"                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
      [2] "<p>See <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie (2020b)<span class=\"marginnote\">Xie, Yihui. 2020b. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r - Duplicate</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>.</span></span></p>"                                                                                                                                                                                                                                                                                                                                                                                                                                                                    
      [3] "<p>See <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie (2020a)<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>.</span></span> and <span class=\"citation\"><label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie, Allaire, and Grolemund (2018)<span class=\"marginnote\">Xie, Yihui, J. J. Allaire, and Garrett Grolemund. 2018. <em>R Markdown: The Definitive Guide</em>. Boca Raton, Florida: Chapman; Hall/CRC. <a href=\"https://bookdown.org/yihui/rmarkdown\">https://bookdown.org/yihui/rmarkdown</a>.</span></span></p>"
      [4] "<p>See <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie 2020a<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in r</em>. <a href=\"https://yihui.org/knitr/\">https://yihui.org/knitr/</a>.</span>)</span> and <span class=\"citation\">(<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">Xie, Allaire, and Grolemund 2018<span class=\"marginnote\">Xie, Yihui, J. J. Allaire, and Garrett Grolemund. 2018. <em>R Markdown: The Definitive Guide</em>. Boca Raton, Florida: Chapman; Hall/CRC. <a href=\"https://bookdown.org/yihui/rmarkdown\">https://bookdown.org/yihui/rmarkdown</a>.</span>)</span></p>"

# add marginnote

    Code
      marginnote_html("text")
    Output
      [1] "<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">text"

---

    Code
      marginnote_html("text", "#")
    Output
      [1] "<label for=\"tufte-mn-\" class=\"margin-toggle\">#</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">text"

# put references in margin when link-citations: yes

    Code
      margin_references(x)
    Output
      [1] "<p>See <span class=\"citation\">(Xie <label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">2020a)<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in R</em>.</span></span>.</p>"                                                                                                                                                                                                                                                                                                                                                                                    
      [2] "<p>See <span class=\"citation\">Xie (<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">2020b)<span class=\"marginnote\">Xie, Yihui. 2020b. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in R - Duplicate</em>.</span></span></p>"                                                                                                                                                                                                                                                                                                                                                                         
      [3] "<p>See <span class=\"citation\">Xie (<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">2020a)<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in R</em>.</span></span> and <span class=\"citation\">Xie, Allaire, and Grolemund (<label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">2018<span class=\"marginnote\">Xie, Yihui, J. J. Allaire, and Garrett Grolemund. 2018. <em>R Markdown: The Definitive Guide</em>. Boca Raton, Florida: Chapman; Hall/CRC.</span>)</span></p>"
      [4] "<p>See <span class=\"citation\">(Xie <label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">2020a)<span class=\"marginnote\">Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in R</em>.</span></span> and <span class=\"citation\">(Xie, Allaire, and Grolemund <label for=\"tufte-mn-\" class=\"margin-toggle\">&#8853;</label><input type=\"checkbox\" id=\"tufte-mn-\" class=\"margin-toggle\">2018<span class=\"marginnote\">Xie, Yihui, J. J. Allaire, and Garrett Grolemund. 2018. <em>R Markdown: The Definitive Guide</em>. Boca Raton, Florida: Chapman; Hall/CRC.</span>)</span></p>"

---

    Code
      margin_references(x)
    Output
       [1] "<p>See <span class=\"citation\">(Xie 2020a)</span>.</p>"                                                                                          
       [2] "<p>See <span class=\"citation\">Xie (2020b)</span></p>"                                                                                           
       [3] "<p>See <span class=\"citation\">Xie (2020a)</span> and <span class=\"citation\">Xie, Allaire, and Grolemund (2018)</span></p>"                    
       [4] "<p>See <span class=\"citation\">(Xie 2020a)</span> and <span class=\"citation\">(Xie, Allaire, and Grolemund 2018)</span></p>"                    
       [5] "<div id=\"refs\" class=\"references\">"                                                                                                           
       [6] "<div id=\"ref-R-knitr\">"                                                                                                                         
       [7] "<p>Xie, Yihui. 2020a. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in R</em>.</p>"                                          
       [8] "</div>"                                                                                                                                           
       [9] "<div id=\"ref-R-knitr2\">"                                                                                                                        
      [10] "<p>———. 2020b. <em>Knitr: A General-Purpose Package for Dynamic Report Generation in R - Duplicate</em>.</p>"                                     
      [11] "</div>"                                                                                                                                           
      [12] "<div id=\"ref-rmarkdown2018\">"                                                                                                                   
      [13] "<p>Xie, Yihui, J. J. Allaire, and Garrett Grolemund. 2018. <em>R Markdown: The Definitive Guide</em>. Boca Raton, Florida: Chapman; Hall/CRC.</p>"
      [14] "</div>"                                                                                                                                           
      [15] "</div>"                                                                                                                                           

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/handout.R, R/html.R, R/utils.R
\name{tufte_handout}
\alias{tufte_handout}
\alias{tufte_book}
\alias{tufte_handout2}
\alias{tufte_book2}
\alias{tufte_html2}
\alias{tufte_html}
\alias{newthought}
\alias{margin_note}
\alias{quote_footer}
\alias{sans_serif}
\title{Tufte handout formats (PDF and HTML)}
\usage{
tufte_handout(
  fig_width = 4,
  fig_height = 2.5,
  fig_crop = "auto",
  dev = "pdf",
  highlight = "default",
  margin_fig_pos = NULL,
  ...
)

tufte_book(
  fig_width = 4,
  fig_height = 2.5,
  fig_crop = "auto",
  dev = "pdf",
  highlight = "default",
  margin_fig_pos = NULL,
  ...
)

tufte_handout2(...)

tufte_book2(...)

tufte_html2(...)

tufte_html(
  ...,
  tufte_features = c("fonts", "background", "italics"),
  tufte_variant = c("default", "envisioned"),
  margin_references = TRUE
)

newthought(text)

margin_note(text, icon = "&#8853;")

quote_footer(text)

sans_serif(text)
}
\arguments{
\item{fig_width}{Default width (in inches) for figures}

\item{fig_height}{Default height (in inches) for figures}

\item{fig_crop}{Whether to crop PDF figures with the command
\command{pdfcrop}. This requires the tools \command{pdfcrop} and
\command{ghostscript} to be installed. By default, \code{fig_crop = TRUE}
if these two tools are available.}

\item{dev}{Graphics device to use for figure output (defaults to pdf)}

\item{highlight}{Syntax highlighting style passed to Pandoc.

 Supported built-in styles include "default", "tango", "pygments", "kate",
 "monochrome", "espresso", "zenburn", "haddock", and "breezedark".

  Two custom styles are also included, "arrow", an accessible color scheme,
  and "rstudio", which mimics the default IDE theme. Alternatively, supply a
  path to a \samp{.theme} file to use
  \href{https://pandoc.org/MANUAL.html#syntax-highlighting}{a custom Pandoc
  style}. Note that custom theme requires Pandoc 2.0+.

  Pass \code{NULL} to prevent syntax highlighting.}

\item{margin_fig_pos}{Default vertical offset for margin figures (a LaTeX
length such as \code{"0cm"} or \code{"-5pt"}). When set, this value is used as the
\code{fig.pos} for all chunks with \code{fig.margin = TRUE}, unless the chunk
specifies its own \code{fig.pos}. This avoids the problem of setting \code{fig.pos}
globally via \code{knitr::opts_chunk$set()}, which would also affect regular
figures where \code{fig.pos} is a placement specifier (e.g. \code{"htbp"}), not a
length. Can also be set per-chunk or via \code{knitr::opts_chunk$set()} as the
\code{margin_fig_pos} chunk option. Defaults to \code{NULL} (use the Tufte-LaTeX
class default of \verb{-1.2ex}).}

\item{...}{Other arguments to be passed to \code{\link[rmarkdown:pdf_document]{rmarkdown::pdf_document()}} or
\code{\link[rmarkdown:html_document]{rmarkdown::html_document()}} (note you cannot use the \code{template}
argument in \code{tufte_handout} or the \code{theme} argument in
\code{tufte_html()}; these arguments have been set internally)}

\item{tufte_features}{A character vector of style features to enable:
\code{fonts} stands for the \code{et-book} fonts in the \code{tufte-css}
project, \code{background} means the lightyellow background color of the
page, and \code{italics} means whether to use italics for the headers. You
can enable a subset of these features, or just disable all of them by
\code{NULL}. When this argument is not used and the \code{tufte_variant}
argument is not \code{default}, no features are enabled.}

\item{tufte_variant}{A variant of the Tufte style. Currently supported styles
are \code{default} (from the \code{tufte-css} project), and
\code{envisioned} (inspired by the project \verb{Envisioned CSS}
\url{https://github.com/nogginfuel/envisioned-css} but essentially just
sets the font family to \verb{Roboto Condensed}, and changed the
background/foreground colors).}

\item{margin_references}{Whether to place citations in margin notes.}

\item{text}{A character string to be presented as a \dQuote{new thought}
(using small caps), or a margin note, or a footer of a quote}

\item{icon}{A character string to indicate there is a hidden margin note when
the page width is too narrow (by default it is a circled plus sign)}
}
\value{
\code{tufte_handout()}, \code{tufte_book()}, \code{tufte_html()}, and the
bookdown wrappers \code{tufte_handout2()}, \code{tufte_book2()}, and
\code{tufte_html2()} return an R Markdown output format object that can
be passed to \code{\link[rmarkdown:render]{rmarkdown::render()}}. The inline helpers
\code{newthought()}, \code{margin_note()}, \code{quote_footer()}, and
\code{sans_serif()} return a character string with HTML or LaTeX markup
depending on the active output format.
}
\description{
Templates for creating handouts according to the style of Edward R. Tufte and
Richard Feynman.
}
\details{
\code{tufte_handout()} provides the PDF format based on the Tufte-LaTeX
class: \url{https://tufte-latex.github.io/tufte-latex/}.

\code{tufte_handout2()} and \code{tufte_book2()} are wrappers around
\code{\link[bookdown:pdf_book]{bookdown::pdf_book()}} that use the corresponding tufte format as the
base format. They support bookdown text references (\code{(ref:label)}) in
figure and table captions, cross-references, and other bookdown
extensions. Requires the \pkg{bookdown} package to be installed.

\code{tufte_html()} provides the HTML format based on the Tufte CSS:
\url{https://edwardtufte.github.io/tufte-css/}.

\code{tufte_html2()} is a wrapper around \code{\link[bookdown:html_document2]{bookdown::html_document2()}}
that uses \code{tufte_html()} as the base format, enabling text references
and cross-references. Requires the \pkg{bookdown} package.

\code{newthought()} can be used in inline R expressions in R
Markdown

\if{html}{\out{<div class="sourceCode r">}}\preformatted{`r newthought(Some text)`
}\if{html}{\out{</div>}}

and it works for both
HTML (\samp{<span class="newthought">text</span>}) and PDF
(\samp{\\newthought{text}}) output.

\code{margin_note()} can be used in inline R expressions to write a
margin note (like a sidenote but not numbered).

\code{quote_footer()} formats text as the footer of a quote. It works in
any HTML or LaTeX output format, not only tufte formats, and tries to
render \code{text} consistently across formats:
\itemize{
\item If \code{text} starts with \code{"---"} (or an em-dash), an em-dash + non-breaking
space is rendered before the rest of the text in every output. The
leading \code{"---"} is stripped from \code{text} to avoid duplication when the
destination format would otherwise inject its own em-dash.
\item For tufte HTML, \code{text} is wrapped in a \verb{<footer>} element styled by
\code{tufte.css}. For non-tufte HTML (e.g. \code{rmarkdown::html_document()},
including \code{bslib} Bootstrap 4 and 5 themes), \code{text} is wrapped in a
\verb{<footer class="blockquote-footer">} so that Bootstrap's
\code{.blockquote-footer} styling (including the \verb{::before} em-dash) applies.
An inline \code{text-align: right} is added in both cases.
\item For LaTeX output (tufte or non-tufte), \code{text} is prepended with
\samp{\\hfill} to right-align it. Pandoc's smart-punctuation extension
converts a leading \code{"---"} to an em-dash glyph.
}

\code{quote_footer()} detects whether the active output is a tufte format via
the \code{tufte.format} entry that the tufte output formats register in
\code{knitr::opts_knit}.

\code{sans_serif()} applies sans-serif fonts to \code{text}.
}
\examples{
library(tufte)
newthought("In this section")
}
\references{
See \url{https://rstudio.github.io/tufte/} for an example.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tufte-package.R
\docType{package}
\name{tufte-package}
\alias{tufte}
\alias{tufte-package}
\title{tufte: Tufte's Styles for R Markdown Documents}
\description{
Provides R Markdown output formats to use Tufte styles for PDF and HTML output.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/rstudio/tufte}
  \item Report bugs at \url{https://github.com/rstudio/tufte/issues}
}

}
\author{
\strong{Maintainer}: Christophe Dervieux \email{cderv@posit.co} (\href{https://orcid.org/0000-0003-4474-2498}{ORCID}) [contributor]

Authors:
\itemize{
  \item Yihui Xie \email{xie@yihui.name} (\href{https://orcid.org/0000-0003-0645-5666}{ORCID})
  \item JJ Allaire
}

Other contributors:
\itemize{
  \item Andrzej Oles [contributor]
  \item Dave Liepmann (Tufte CSS in inst/rmarkdown/templates/tufte_html/resources) [contributor]
  \item Posit Software, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}
